@@ -43,6 +43,7 @@ class CAASymCipher
 			SINT32 encryptOAEP(const UINT8* from,UINT32 fromlen,UINT8* to,UINT32* len);
 			SINT32 generateKeyPair(UINT32 size);
 			//SINT32 getPublicKey(UINT8* buff,UINT32 *len);
+#ifndef ONLY_LOCAL_PROXY
 			SINT32 getPublicKeyAsXML(UINT8* buff,UINT32* len);
 			SINT32 getPublicKeyAsDocumentFragment(DOM_DocumentFragment& pDFrag);
 			//SINT32 getPublicKeySize();
@@ -50,6 +51,9 @@ class CAASymCipher
 			SINT32 setPublicKey(const CACertificate* pCert);
 			SINT32 setPublicKeyAsXML(const UINT8* buff,UINT32 len);
 			SINT32 setPublicKeyAsDOMNode(DOM_Node& node);
+#endif
+			//Set the public key from a Base64 encodes exponent and modulus
+			SINT32 setPublicKey(const UINT8* modulus,UINT32 moduluslen,const UINT8* exponent,UINT32 exponentlen);
 		private:
 			RSA* m_pRSA;
 	};
