@@ -26,6 +26,7 @@ IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISI
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 */
 #include "StdAfx.h"
+#ifndef ONLY_LOCAL_PROXY
 #include "CAAbstractXMLEncodable.hpp"
 #include "xml/DOM_Output.hpp"
 
@@ -51,3 +52,4 @@ UINT8* CAAbstractXMLEncodable::toXmlString(UINT32 &size)
 		delete[] tmp;
 		return tmp2;
 	}
+#endif //ONLY_LOCAL_PROXY
