@@ -25,10 +25,11 @@ OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABIL
 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 */
-#ifdef PAYMENT
+
 #ifndef __CAACCOUNTINGINSTANCE__
 #define __CAACCOUNTINGINSTANCE__
-
+#include "doxygen.h"
+#ifdef PAYMENT
 #include "CAFirstMixChannelList.hpp"
 #include "CASymCipher.hpp"
 #include "CAQueue.hpp"
@@ -49,6 +50,7 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMA
 // the number of seconds that may pass between a pay request
 // and the jap sending its answer
 #define REQUEST_TIMEOUT 30
+#define HARD_LIMIT_TIMEOUT 30
 #define MIN_BALANCE 1024*512
 
 
