@@ -26,6 +26,7 @@ IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISI
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 */
 #include "StdAfx.h"
+#ifndef ONLY_LOCAL_PROXY
 #include "CACertStore.hpp"
 
 CACertStore::CACertStore()
@@ -114,3 +115,4 @@ SINT32 CACertStore::encode(DOM_DocumentFragment& docFrag,DOM_Document& doc)
 			}
 		return E_SUCCESS;
 	}
+#endif //ONLY_LOCAL_PROXY
