@@ -27,9 +27,11 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMA
 */
 
 #include "StdAfx.h"
+#ifndef ONLY_LOCAL_PROXY
 #include "CACertificate.hpp"
 #include "CABase64.hpp"
 #include "CAUtil.hpp"
+#include "CAMsg.hpp"
 
 UINT8 * CACertificate::m_spXmlElementName =0;
 
@@ -104,14 +106,14 @@ CACertificate* CACertificate::decode(const UINT8* buff,UINT32 bufflen,UINT32 typ
 			{
 				case CERT_DER:
 					tmp=buff;
-					#if OPENSSL_VERSION_NUMBER	> 0x0090705fL
+					#if OPENSSL_VERSION_NUMBER	> 0x009070CfL
 						tmpCert=d2i_X509(NULL,&tmp,bufflen);
 					#else
 						tmpCert=d2i_X509(NULL,(UINT8**)&tmp,bufflen);
 					#endif
 				break;
 				case CERT_PKCS12:
-					#if OPENSSL_VERSION_NUMBER	> 0x0090705fL
+					#if OPENSSL_VERSION_NUMBER	> 0x009070CfL
 						tmpPKCS12=d2i_PKCS12(NULL,&buff,bufflen);	
 					#else
 						tmpPKCS12=d2i_PKCS12(NULL,(UINT8**)&buff,bufflen);	
@@ -132,7 +134,7 @@ CACertificate* CACertificate::decode(const UINT8* buff,UINT32 bufflen,UINT32 typ
 					getDOMElementValue(root,tmpBuff,&tmpBuffSize);
 					CABase64::decode(tmpBuff,tmpBuffSize,tmpBuff,&tmpBuffSize);
 					tmp=tmpBuff;
-					#if OPENSSL_VERSION_NUMBER	> 0x0090705fL
+					#if OPENSSL_VERSION_NUMBER	> 0x009070CfL
 						tmpCert=d2i_X509(NULL,&tmp,tmpBuffSize);
 					#else
 						tmpCert=d2i_X509(NULL,(UINT8**)&tmp,tmpBuffSize);
@@ -194,3 +196,125 @@ SINT32 CACertificate::encode(DOM_DocumentFragment& docFrag,DOM_Document& doc)
 		setDOMElementValue(elemCert,buff);
 		return E_SUCCESS;
 	}
+
+/**
+  * LERNGRUPPE
+  * Accessor method for the subjectKeyIdentifier (SKI) extension stored in this certificate
+  * @return r_ski The SKI as colon-free string
+  * @return r_skiLen The length of r_ski
+  * @retval E_SUCCESS upon successful retrieval
+  * @retval E_UNKNOWN otherwise
+  */
+SINT32 CACertificate::getSubjectKeyIdentifier(UINT8* r_ski, UINT32 *r_skiLen)
+{
+    // FIXME Do we have a memory leak here? Maybe we need to ASN1_OCTET_STRING_free pSkid?
+    ASN1_OCTET_STRING *pSki = NULL;
+    pSki = (ASN1_OCTET_STRING*)X509_get_ext_d2i(m_pCert, NID_subject_key_identifier, NULL, NULL );
+    if ( !pSki )
+    {
+#ifdef DEBUG
+        CAMsg::printMsg( LOG_ERR, "Unable to get SKI from Certificate, trying to recover\n");
+#endif
+        setSubjectKeyIdentifier();
+        pSki = (ASN1_OCTET_STRING*)X509_get_ext_d2i(m_pCert, NID_subject_key_identifier, NULL, NULL );
+        if( ! pSki ) 
+        {
+            CAMsg::printMsg( LOG_ERR, "Unable to retrieve SKI from Certificate\n");
+            return E_UNKNOWN;
+        } 
+#ifdef DEBUG
+        else
+        {
+            CAMsg::printMsg( LOG_ERR, "Recovery SUCCESSFUL!\n");
+        }
+#endif
+    }
+    // Get the ASCII string format of the subject key identifier
+    UINT8* cSki = (UINT8*)i2s_ASN1_OCTET_STRING( NULL, pSki );
+    if ( !cSki )
+    {
+        CAMsg::printMsg( LOG_ERR, "Unable to convert SKI\n");
+        return E_UNKNOWN;
+    }
+#ifdef DEBUG
+    CAMsg::printMsg( LOG_ERR, "getSubjectKeyIdentifier: SKI is %s\n", cSki);
+#endif
+    removeColons(cSki, strlen((const char*)cSki), r_ski, r_skiLen);
+    return E_SUCCESS;
+}
+
+/**
+  * LERNGRUPPE
+  * Removes the colons from the string representation of the given SKI
+  * @param a_cSkid The string from which the colons should be removed
+  * @param a_cSkidLen The length of a_cSkid
+  * @param r_ski 
+  * @param r_skiLen 
+  * @return r_ski The SKI as colon-free string
+  * @return r_skiLen The length of r_ski
+  * @retval E_SUCCESS upon successful removal
+  * @retval E_UNKNOWN otherwise
+  */
+SINT32 CACertificate::removeColons(UINT8* a_cSkid, UINT32 a_cSkidLen, UINT8 *&r_ski, UINT32 *r_skiLen)
+{
+    UINT32 i = 0, j = 0;
+    UINT32 tmp = (2*a_cSkidLen)/3 + 2;
+    if(*r_skiLen < tmp)
+    {
+        CAMsg::printMsg( LOG_ERR, "Unable to copy SKI to target array, size must at least be %i but is only %i!\n", tmp, *r_skiLen);
+        return E_UNKNOWN;
+    }
+    for(i = 0; i < a_cSkidLen; i++)
+    {
+        if(i % 3 == 2) {
+            j++;
+            continue;
+        }
+        r_ski[i-j] = a_cSkid[i];
+    }
+    r_ski[i-j] = 0;
+    *r_skiLen = i-j-1;
+    return E_SUCCESS;
+}
+
+/**
+  * LERNGRUPPE
+  * Sets the subjectKeyIdentifier extension for this certificate to the hash of the public key
+  * @retval E_SUCCESS upon successful removal
+  * @retval E_UNKNOWN otherwise
+  */
+SINT32 CACertificate::setSubjectKeyIdentifier() 
+{
+    UINT32 len = 0;
+    UINT8 sha_hash[SHA_DIGEST_LENGTH];
+    X509_pubkey_digest(m_pCert, EVP_sha1(), sha_hash, &len);
+    return setSubjectKeyIdentifier( sha_hash, len );
+}
+
+/**
+  * LERNGRUPPE
+  * Sets the subjectKeyIdentifier extension for this certificate to the given value
+  * @param a_value The value which should be set as SKI
+  * @param a_valueLen The length of a_value
+  * @retval E_SUCCESS upon successful removal
+  * @retval E_UNKNOWN otherwise
+  */
+SINT32 CACertificate::setSubjectKeyIdentifier( UINT8* a_value, UINT32 a_valueLen )
+{
+    SINT32 ret = E_UNKNOWN;
+    ASN1_OCTET_STRING* skid = NULL;
+
+    skid = ASN1_OCTET_STRING_new();
+    if(NULL == skid)   goto end;
+
+    ASN1_OCTET_STRING_set(skid, a_value, a_valueLen);
+    if( X509_add1_ext_i2d(m_pCert, NID_subject_key_identifier, skid, false, X509V3_ADD_REPLACE) == 1)
+        ret = E_SUCCESS;
+
+end:
+    ASN1_OCTET_STRING_free(skid);
+    return ret;
+}
+
+#endif //ONLY_LOCAL_PROXY
+
