@@ -47,6 +47,12 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMA
 #define TARGET_HTTP_PROXY		2
 #define TARGET_SOCKS_PROXY	3
 
+// LERNGRUPPE moved this define from CACmdLnOptions.cpp
+#define DEFAULT_TARGET_PORT 6544
+#define DEFAULT_CONFIG_FILE "default.xml"
+#define MIN_INFOSERVICES 1
+// END LERNGRUPPE
+
 struct t_TargetInterface
 	{
 		UINT32 target_type;
@@ -66,7 +72,6 @@ class CACmdLnOptions
 			SINT32 cleanup();
 			void clean();
 			SINT32 parse(int argc,const char** arg);
-			SINT32 reread(CAMix* pMix);
 	    bool getDaemon();
       //bool getProxySupport();
 
@@ -80,6 +85,7 @@ class CACmdLnOptions
 //			SINT32 getServerRTTPort();
 			UINT16 getSOCKSServerPort();
 
+
 			UINT32 getListenerInterfaceCount(){return m_cnListenerInterfaces;}
 			CAListenerInterface* getListenerInterface(UINT32 nr)
 				{
@@ -120,6 +126,7 @@ class CACmdLnOptions
 						return E_UNKNOWN;
 				};
 
+#ifndef ONLY_LOCAL_PROXY
 			//for last Mixes: number of outside visible addresses
 			UINT32 getVisibleAddressesCount(){return m_cnVisibleAddresses;}
 
@@ -135,30 +142,52 @@ class CACmdLnOptions
 
 			UINT16 getSOCKSPort();
 	    SINT32 getSOCKSHost(UINT8* host,UINT32 len);
-	    UINT16 getInfoServerPort();
-	    SINT32 getInfoServerHost(UINT8* host,UINT32 len);
+			CAListenerInterface** getInfoServices(UINT32& r_size);
+#endif //ONLY_LOCAL_PROXY
 
 			SINT32 getMaxOpenFiles()
 				{
 					return m_nrOfOpenFiles;
 				}
 
+
+#ifndef ONLY_LOCAL_PROXY
 			CASignature* getSignKey()
 				{
 					if(m_pSignKey!=NULL)
 						return m_pSignKey->clone();
 					return NULL;
 				}
-
 			/** Returns a COPY of the public test certifcate for that mix.
 				* @retval a COPY of the mix test certifcate.
 				*/
 			CACertificate* getOwnCertificate() const
 				{
 					if(m_pOwnCertificate!=NULL)
+					{
 						return m_pOwnCertificate->clone();
+					}
 					return NULL;
 				}
+				
+			/** Returns a COPY of the Operator Certificate that mix.
+				* @return opCerts
+				*/
+			CACertificate** getOpCertificates(UINT32& r_length) const
+			{
+				if(m_OpCerts!=NULL && m_OpCertsLength > 0)
+				{
+					CACertificate** opCerts = new CACertificate*[m_OpCertsLength];
+					for (UINT32 i = 0; i < m_OpCertsLength; i++)
+					{
+						opCerts[i] = m_OpCerts[i]->clone();
+					}
+					r_length = m_OpCertsLength;
+					return opCerts;
+				}
+				r_length = 0;
+				return NULL;
+			}
 
 			bool hasPrevMixTestCertificate()
 				{
@@ -184,12 +213,15 @@ class CACmdLnOptions
 						return m_pNextMixCertificate->clone();
 					return NULL;
 				}
-
      /** Returns if the encrpyted Log could/should be used**/
 			bool isEncryptedLogEnabled()
 			{
 				return m_bIsEncryptedLogEnabled;
 			}
+			bool isSyslogEnabled()
+			{
+				return m_bSyslog;
+			}
 
 			/** Set to true if the encrpyted log could/should be used**/
 			SINT32 enableEncryptedLog(bool b)
@@ -222,27 +254,48 @@ class CACmdLnOptions
 				strcpy(m_strCascadeName,name);
 				return E_SUCCESS;
 			}
-    
-			SINT32 getLogDir(UINT8* name,UINT32 len);
+
+			SINT32 reread(CAMix* pMix);
+
+
 			SINT32 getEncryptedLogDir(UINT8* name,UINT32 len);
+
+		/** Get the XML describing the Mix. this is not a string!*/
+			//SINT32 getMixXml(UINT8* strxml,UINT32* len);
+			SINT32 getMixXml(DOM_Document &docMixInfo);
+			
+			UINT32 getKeepAliveSendInterval()
+				{
+				return m_u32KeepAliveSendInterval;
+				}
+
+			UINT32 getKeepAliveRecvInterval()
+				{
+				return m_u32KeepAliveRecvInterval;
+				}
+			
+#endif //ONLY_LOCAL_PROXY			
 			bool getCompressLogs()
 				{
 					return m_bCompressedLogs;
 				}
-			
+			SINT32 getLogDir(UINT8* name,UINT32 len);
+			SINT32 setLogDir(const UINT8* name,UINT32 len);
 			SINT32 getUser(UINT8* user,UINT32 len);
 			SINT32 getPidFile(UINT8* pidfile,UINT32 len);
 
-			/** Get the XML describing the Mix. this is not a string!*/
-			//SINT32 getMixXml(UINT8* strxml,UINT32* len);
-			SINT32 getMixXml(DOM_Document &docMixInfo);
+			UINT32 getMaxNrOfUsers()
+			{
+				return m_maxNrOfUsers;
+			}
+
 			bool isLocalProxy();
 			bool isFirstMix();
 			bool isMiddleMix();
 			bool isLastMix();
 			bool isInfoServiceEnabled()
 				{
-					return m_strInfoServerHost!=NULL;
+					return m_addrInfoServicesSize;
 				}
 
 			bool getAutoReconnect()
@@ -250,11 +303,17 @@ class CACmdLnOptions
 					return m_bAutoReconnect;
 				}
 #ifdef LOG_CRIME
-			regex_t* getCrimeRegExps(UINT32* len)
-				{
-					*len=m_nCrimeRegExps;
-					return m_arCrimeRegExps;
-				}
+			regex_t* getCrimeRegExpsURL(UINT32* len)
+			{
+				*len=m_nCrimeRegExpsURL;
+				return m_arCrimeRegExpsURL;
+			}
+			
+			regex_t* getCrimeRegExpsPayload(UINT32* len)
+			{
+				*len=m_nCrimeRegExpsPayload;
+				return m_arCrimeRegExpsPayload;
+			}
 #endif
 
 #if defined(DELAY_CHANNELS)||defined(DELAY_USERS)
@@ -301,7 +360,9 @@ class CACmdLnOptions
 			
 #endif	
 
-    // added by ronin <ronin2@web.de>
+
+#ifndef ONLY_LOCAL_PROXY
+			// added by ronin <ronin2@web.de>
     // needed for autoconfiguration
 			SINT32 setNextMix(DOM_Document&);
 			SINT32 setPrevMix(DOM_Document&);
@@ -311,30 +372,107 @@ class CACmdLnOptions
 			
 			/** Writes a default configuration file into the file named by filename*/
 			static SINT32 createMixOnCDConfiguration(const UINT8* strFileName);
+		static SINT32 saveToFile(DOM_Document a_doc, const UINT8* a_strFileName);
+#ifdef DYNAMIC_MIX
+			/* LERNGRUPPE (refactoring + new) */
+			//SINT32 createMixOnCDConfiguration(const UINT8* strFileName);
+			SINT32 createDefaultConfiguration();
+			SINT32 addListenerInterface(DOM_Element a_elem);
+			SINT32 resetNetworkConfiguration();
+                        SINT32 getRandomInfoService(CASocketAddrINet *&r_address);
+			bool isDynamic() { return m_bDynamic; }
+			SINT32 changeMixType(CAMix::tMixType a_newMixType);
+			SINT32 resetNextMix();
+			SINT32 resetPrevMix();
+			SINT32 setCascadeProposal(UINT8* a_strCascadeProposal, UINT32 a_len)
+			{
+				if(m_strLastCascadeProposal != NULL)
+				{
+ 					delete m_strLastCascadeProposal;
+					m_strLastCascadeProposal = NULL;
+				}
+				if(a_strCascadeProposal == NULL)
+					return E_SUCCESS;
+				m_strLastCascadeProposal = new UINT8[ a_len + 1 ];
+				memcpy(m_strLastCascadeProposal, a_strCascadeProposal, a_len+1);
+				return E_SUCCESS;
+			}
+			SINT32 getLastCascadeProposal(UINT8* r_strCascadeProposal, UINT32 r_len)
+			{
+				if(m_strLastCascadeProposal == NULL)
+				{
+					return E_UNKNOWN;
+				}
+				if(r_len >= strlen((char*)m_strLastCascadeProposal))
+				{
+					r_len = strlen((char*)m_strLastCascadeProposal);
+					memcpy(r_strCascadeProposal, m_strLastCascadeProposal, r_len + 1);
+					return E_SUCCESS;
+				}
+				return E_UNKNOWN;
+			}
+
+#endif // DYNAMIC_MIX
 		private:
+#ifdef DYNAMIC_MIX
+			UINT8* m_strLastCascadeProposal;
+      UINT32 getRandom(UINT32 a_max);
+			SINT32 checkInfoServices(UINT32 *r_runningInfoServices);
+			SINT32 checkMixId();
+			SINT32 checkListenerInterfaces();
+			SINT32 checkCertificates();
+#endif //DYNAMIC_MIX
+			bool m_bDynamic;
+			SINT32 parseInfoServices(DOM_Element a_infoServiceNode);
+			/* END LERNGRUPPE */
+			static SINT32 buildDefaultConfig(DOM_Document a_doc,bool bForLastMix);
+#endif //only_LOCAL_PROXY
 			UINT8*	m_strConfigFile; //the filename of the config file
-			bool		m_bIsRunReConfigure; //true, if an async reconfigure is under way
-	    CAMutex* m_pcsReConfigure; //Ensures that reconfigure is running only once at the same time;
-			CAThread m_threadReConfigure; //Thread, that does the actual reconfigure work
 			bool		m_bDaemon;
 	    UINT16	m_iSOCKSServerPort;
 	    UINT16	m_iTargetPort; //only for the local proxy...
 			char*		m_strTargetHost; //only for the local proxy...
 			char*		m_strSOCKSHost;
 	    UINT16	m_iSOCKSPort;
-	    char*		m_strInfoServerHost;
-	    UINT16	m_iInfoServerPort;
+#ifndef ONLY_LOCAL_PROXY
+			bool		m_bIsRunReConfigure; //true, if an async reconfigure is under way
+	    CAMutex* m_pcsReConfigure; //Ensures that reconfigure is running only once at the same time;
+			CAThread m_threadReConfigure; //Thread, that does the actual reconfigure work
+	    CAListenerInterface**	m_addrInfoServices;
+
+			CASignature*		m_pSignKey;
+			CACertificate*	m_pOwnCertificate;
+			
+			CACertificate** m_OpCerts;
+			UINT32 m_OpCertsLength;
+
+			CACertificate*	m_pPrevMixCertificate;
+			CACertificate*	m_pNextMixCertificate;
+			CACertificate*	m_pLogEncryptionCertificate;
+			
+			UINT32 m_maxNrOfUsers;
+
+			// added by ronin <ronin2@web.de>
+			DOM_Element m_oCascadeXML;
+			bool m_bAcceptReconfiguration;
+			DOM_Document m_docMixInfo;
+			DOM_Document m_docMixXml;
+			
+			UINT32 m_u32KeepAliveSendInterval;
+			UINT32 m_u32KeepAliveRecvInterval;
+#endif //ONLY_LOCAL_PROXY
+
+			UINT32 m_addrInfoServicesSize;
 			bool		m_bLocalProxy,m_bFirstMix,m_bMiddleMix,m_bLastMix;
 			bool		m_bAutoReconnect; //auto reconnect if connection to first mix lost ??
 			char*		m_strCascadeName;
 			char*		m_strLogDir;
 			char*		m_strEncryptedLogDir;
 			bool		m_bCompressedLogs;
+			bool		m_bSyslog;
 			char*		m_strUser;
 			char*		m_strPidFile;
 			SINT32	m_nrOfOpenFiles; //How many open files (sockets) should we use
-			DOM_Document m_docMixInfo;
-			DOM_Document m_docMixXml;
     
 			//char*		m_strMixXml;
 			char*		m_strMixID;
@@ -348,19 +486,12 @@ class CACmdLnOptions
 			UINT8**								m_arStrVisibleAddresses;
 			UINT32								m_cnVisibleAddresses;
 	
-			CASignature*		m_pSignKey;
-			CACertificate*	m_pOwnCertificate;
-			CACertificate*	m_pPrevMixCertificate;
-			CACertificate*	m_pNextMixCertificate;
-			CACertificate*	m_pLogEncryptionCertificate;
-
-    // added by ronin <ronin2@web.de>
-    DOM_Element m_oCascadeXML;
-    bool m_bAcceptReconfiguration;
 
 #ifdef LOG_CRIME
-			regex_t* m_arCrimeRegExps;
-			UINT32 m_nCrimeRegExps;
+			regex_t* m_arCrimeRegExpsURL;
+			UINT32 m_nCrimeRegExpsURL;
+			regex_t* m_arCrimeRegExpsPayload;
+			UINT32 m_nCrimeRegExpsPayload;
 #endif
 #if defined (DELAY_CHANNELS) ||defined(DELAY_USERS)
 		UINT32 m_u32DelayChannelUnlimitTraffic;	
@@ -390,12 +521,14 @@ class CACmdLnOptions
 
 		private:
 			SINT32 setNewValues(CACmdLnOptions& newOptions);
+#ifndef ONLY_LOCAL_PROXY
 			SINT32 readXmlConfiguration(DOM_Document& docConfig,const UINT8* const configFileName);
 			SINT32 readXmlConfiguration(DOM_Document& docConfig,const UINT8* const buf, UINT32 len);
 			SINT32 processXmlConfiguration(DOM_Document& docConfig);
-			SINT32 clearTargetInterfaces();
-			SINT32 clearListenerInterfaces();
 			SINT32 clearVisibleAddresses();
 			SINT32 addVisibleAddresses(DOM_Node& nodeProxy);
+#endif
+			SINT32 clearTargetInterfaces();
+			SINT32 clearListenerInterfaces();
 	};
 #endif
