@@ -26,6 +26,7 @@ IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISI
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 */
 #include "StdAfx.h"
+#ifndef ONLY_LOCAL_PROXY
 #include "CAFirstMix.hpp"
 #include "CASocketGroup.hpp"
 #include "CASingleSocketGroup.hpp"
@@ -83,6 +84,10 @@ SINT32 CAFirstMix::initOnce()
 
 SINT32 CAFirstMix::init()
 	{
+#ifdef DYNAMIC_MIX
+		m_bBreakNeeded = m_bReconfigured;
+#endif
+
 		CAMsg::printMsg(LOG_DEBUG,"Starting FirstMix Init\n");
 		UINT8 buff[255];
 		m_nMixedPackets=0; //reset to zero after each restart (at the moment neccessary for infoservice)
@@ -165,13 +170,11 @@ SINT32 CAFirstMix::init()
 		CAMsg::printMsg(LOG_INFO,"MUXOUT-SOCKET SendBuffSize: %i\n",((CASocket*)(*m_pMuxOut))->getSendBuff());
 		//CAMsg::printMsg(LOG_INFO,"MUXOUT-SOCKET SendLowWatSize: %i\n",((CASocket*)(*m_pMuxOut))->getSendLowWat());
 
-		pAddrNext->toString(buff,255);
-		CAMsg::printMsg(LOG_INFO,"Try to connect to next Mix on %s ...\n",buff);
-
-		if(m_pMuxOut->connect(*pAddrNext,10,10)!=E_SUCCESS)
+		/** Connect to the next mix */
+		if(connectToNextMix(pAddrNext) != E_SUCCESS)
 			{
 				delete pAddrNext;
-				CAMsg::printMsg(LOG_CRIT,"Cannot connect to next Mix!\n");
+			CAMsg::printMsg(LOG_DEBUG, "CAFirstMix::init - Unable to connect to next mix\n");
 				return E_UNKNOWN;
 			}
 		delete pAddrNext;
@@ -243,6 +246,44 @@ SINT32 CAFirstMix::init()
 		return E_SUCCESS;
 }
 
+SINT32 CAFirstMix::connectToNextMix(CASocketAddr* a_pAddrNext)
+{
+	UINT8 buff[255];
+	a_pAddrNext->toString(buff,255);
+	CAMsg::printMsg(LOG_INFO,"Try to connect to next Mix on %s ...\n",buff);
+	SINT32 err = E_UNKNOWN;
+	for(UINT32 i=0; i < 100; i++)
+	{
+#ifdef DYNAMIC_MIX
+		/** If someone reconfigured the mix, we need to break the loop as the next mix might have changed */
+		if(m_bBreakNeeded != m_bReconfigured)
+		{
+			CAMsg::printMsg(LOG_DEBUG, "CAFirstMix::connectToNextMix - Broken the connect loop!\n");
+			break;
+		}
+#endif
+		err = m_pMuxOut->connect(*a_pAddrNext);
+		if(err != E_SUCCESS)
+		{
+			err=GET_NET_ERROR;
+#ifdef _DEBUG
+		 	CAMsg::printMsg(LOG_DEBUG,"Con-Error: %i\n",err);
+#endif
+			if(err!=ERR_INTERN_TIMEDOUT&&err!=ERR_INTERN_CONNREFUSED)
+				break;
+#ifdef _DEBUG
+			CAMsg::printMsg(LOG_DEBUG,"Cannot connect... retrying\n");
+#endif				
+			sSleep(10);
+		}
+		else
+		{
+			break;
+		}
+	}
+	return err;
+}
+
 /*
 *@DOCME
 */
@@ -280,7 +321,7 @@ SINT32 CAFirstMix::processKeyExchange()
 		DOM_Document doc=oParser.getDocument();
     DOM_Element elemMixes=doc.getDocumentElement();
     if(elemMixes==NULL)
-        return E_UNKNOWN;
+		return E_UNKNOWN;
 		SINT32 count=0;
     if(getDOMElementAttribute(elemMixes,"count",&count)!=E_SUCCESS)
         return E_UNKNOWN;
@@ -300,7 +341,7 @@ SINT32 CAFirstMix::processKeyExchange()
 
     DOM_Node child=elemMixes.getLastChild();
 
-    //tmp XML-Structure for constructing the XML which is send to each user
+    //tmp XML-Structure for constructing the XML which is sent to each user
     DOM_Document docXmlKeyInfo=DOM_Document::createDocument();
     DOM_Element elemRootKey=docXmlKeyInfo.createElement("MixCascade");
     setDOMElementAttribute(elemRootKey,"version",(UINT8*)"0.2"); //set the Version of the XML to 0.2
@@ -329,14 +370,18 @@ SINT32 CAFirstMix::processKeyExchange()
     //Inserting own Key in XML-Key struct
     DOM_DocumentFragment docfragKey;
     m_pRSA->getPublicKeyAsDocumentFragment(docfragKey);
-    DOM_Element elemOwnMix=docXmlKeyInfo.createElement("Mix");
-    UINT8 buffId[255];
-    options.getMixId(buffId,255);
-    elemOwnMix.setAttribute("id",DOMString((char*)buffId));
+    addMixInfo(elemMixesKey, true);
+    DOM_Element elemOwnMix;
+    	getDOMChildByName(elemMixesKey, (UINT8*)"Mix", elemOwnMix, false);
 		elemOwnMix.appendChild(docXmlKeyInfo.importNode(docfragKey,true));
-    elemMixesKey.insertBefore(elemOwnMix,elemMixesKey.getFirstChild());
+	if (signXML(elemOwnMix) != E_SUCCESS)
+	{
+		CAMsg::printMsg(LOG_DEBUG,"Could not sign MixInfo sent to users...\n");
+	}
+	
     setDOMElementAttribute((DOM_Element&)elemMixesKey,"count",count+1);
     
+    
 	  DOM_Node elemPayment=docXmlKeyInfo.createElement("Payment");
 		elemRootKey.appendChild(elemPayment);
 		#ifdef PAYMENT
@@ -345,22 +390,13 @@ SINT32 CAFirstMix::processKeyExchange()
 			setDOMElementAttribute(elemPayment,"required",(UINT8*)"false");
 		#endif
 
-		//dign the xml struct send to each jap user
-		CACertificate* ownCert=options.getOwnCertificate();
-    if(ownCert==NULL)
+	// create signature
+	if (signXML(elemRootKey) != E_SUCCESS)
     {
-        CAMsg::printMsg(LOG_DEBUG,"Own Test Cert is NULL -- so it could not be inserted into signed KeyInfo send to users...\n");
-	}
-    CACertStore* tmpCertStore=new CACertStore();
-    tmpCertStore->add
-    (ownCert);
-    if(m_pSignature->signXML(elemRootKey,tmpCertStore)!=E_SUCCESS)
-    {
-        CAMsg::printMsg(LOG_DEBUG,"Could not sign KeyInfo send to users...\n");
-    }
-    delete ownCert;
-    delete tmpCertStore;
+		CAMsg::printMsg(LOG_DEBUG,"Could not sign KeyInfo sent to users...\n");
+		}
 
+    
     UINT32 tlen=0;
     UINT8* tmpB=DOM_Output::dumpToMem(docXmlKeyInfo,&tlen);
     m_xmlKeyInfoBuff=new UINT8[tlen+2];
@@ -424,6 +460,34 @@ SINT32 CAFirstMix::processKeyExchange()
             }
             UINT32 outlen=5000;
             UINT8* out=new UINT8[outlen];
+						///Getting the KeepAlive Traffice...
+						DOM_Element elemKeepAlive;
+						DOM_Element elemKeepAliveSendInterval;
+						DOM_Element elemKeepAliveRecvInterval;
+						getDOMChildByName(child,(UINT8*)"KeepAlive",elemKeepAlive,false);
+						getDOMChildByName(elemKeepAlive,(UINT8*)"SendInterval",elemKeepAliveSendInterval,false);
+						getDOMChildByName(elemKeepAlive,(UINT8*)"ReceiveInterval",elemKeepAliveRecvInterval,false);
+						UINT32 tmpSendInterval,tmpRecvInterval;
+						getDOMElementValue(elemKeepAliveSendInterval,tmpSendInterval,0xFFFFFFFF); //if now send interval was given set it to "infinite"
+						getDOMElementValue(elemKeepAliveRecvInterval,tmpRecvInterval,0xFFFFFFFF); //if no recv interval was given --> set it to "infinite"
+						CAMsg::printMsg(LOG_DEBUG,"KeepAlive-Traffic: Getting offer -- SendInterval %u -- ReceiveInterval %u\n",tmpSendInterval,tmpRecvInterval);
+						// Add Info about KeepAlive traffic
+						UINT32 u32KeepAliveSendInterval=options.getKeepAliveSendInterval();
+						UINT32 u32KeepAliveRecvInterval=options.getKeepAliveRecvInterval();
+						elemKeepAlive=docSymKey.createElement("KeepAlive");
+						elemKeepAliveSendInterval=docSymKey.createElement("SendInterval");
+						elemKeepAliveRecvInterval=docSymKey.createElement("ReceiveInterval");
+						elemKeepAlive.appendChild(elemKeepAliveSendInterval);
+						elemKeepAlive.appendChild(elemKeepAliveRecvInterval);
+						setDOMElementValue(elemKeepAliveSendInterval,u32KeepAliveSendInterval);
+						setDOMElementValue(elemKeepAliveRecvInterval,u32KeepAliveRecvInterval);
+						elemRoot.appendChild(elemKeepAlive);
+						CAMsg::printMsg(LOG_DEBUG,"KeepAlive-Traffic: Offering -- SendInterval %u -- Receive Interval %u\n",u32KeepAliveSendInterval,u32KeepAliveRecvInterval);		
+						m_u32KeepAliveSendInterval=max(u32KeepAliveSendInterval,tmpRecvInterval);
+						if(m_u32KeepAliveSendInterval>10000)
+							m_u32KeepAliveSendInterval-=10000; //make the send interval a little bit smaller than the related receive intervall
+						m_u32KeepAliveRecvInterval=max(u32KeepAliveRecvInterval,tmpSendInterval);
+						CAMsg::printMsg(LOG_DEBUG,"KeepAlive-Traffic: Calculated -- SendInterval %u -- Receive Interval %u\n",m_u32KeepAliveSendInterval,m_u32KeepAliveRecvInterval);		
 
             m_pSignature->signXML(elemRoot);
             DOM_Output::dumpToMem(docSymKey,out,&outlen);
@@ -487,11 +551,18 @@ THREAD_RETURN fm_loopSendToMix(void* param)
 #ifndef USE_POOL
 		tQueueEntry* pQueueEntry=new tQueueEntry;
 		MIXPACKET* pMixPacket=&pQueueEntry->packet;
+		UINT32 u32KeepAliveSendInterval=pFirstMix->m_u32KeepAliveSendInterval;
 		while(!pFirstMix->m_bRestart)
 			{
 				len=sizeof(tQueueEntry);
-				ret=pQueue->getOrWait((UINT8*)pQueueEntry,&len);
-				if(ret!=E_SUCCESS||len!=sizeof(tQueueEntry))
+				ret=pQueue->getOrWait((UINT8*)pQueueEntry,&len,u32KeepAliveSendInterval);
+				if(ret==E_TIMEDOUT)
+					{//send a dummy as keep-alvie-traffic
+						pMixPacket->flags=CHANNEL_DUMMY;
+						pMixPacket->channel=DUMMY_CHANNEL;
+						getRandom(pMixPacket->data,DATA_SIZE);
+					}
+				else if(ret!=E_SUCCESS||len!=sizeof(tQueueEntry))
 					{
 						CAMsg::printMsg(LOG_ERR,"CAFirstMix::lm_loopSendToMix - Error in dequeueing MixPaket\n");
 						CAMsg::printMsg(LOG_ERR,"ret=%i len=%i\n",ret,len);
@@ -570,14 +641,27 @@ THREAD_RETURN fm_loopReadFromMix(void* pParam)
 		#ifdef USE_POOL
 			CAPool* pPool=new CAPool(MIX_POOL_SIZE);
 		#endif
+		UINT64 keepaliveNow,keepaliveLast;
+		UINT32 u32KeepAliveRecvInterval=pFirstMix->m_u32KeepAliveRecvInterval;
+		getcurrentTimeMillis(keepaliveLast);
 		CAControlChannelDispatcher* pControlChannelDispatcher=pFirstMix->m_pMuxOutControlChannelDispatcher;
 		while(!pFirstMix->m_bRestart)
 			{
 				if(pQueue->getSize()>MAX_READ_FROM_NEXT_MIX_QUEUE_SIZE)
 					{
 						msSleep(200);
+						getcurrentTimeMillis(keepaliveLast);
 						continue;
 					}
+				//check if the connection is broken because we did not received a Keep_alive-Message
+				getcurrentTimeMillis(keepaliveNow);
+				UINT32 keepaliveDiff=diff64(keepaliveNow,keepaliveLast);
+				if(keepaliveDiff>u32KeepAliveRecvInterval)
+					{
+						CAMsg::printMsg(LOG_DEBUG,"CAFirstMix::loopReadFromMix() -- restart because of KeepAlive-Traffic Timeout!\n");
+						pFirstMix->m_bRestart=true;
+						break;
+					}
 				SINT32 ret=pSocketGroup->select(MIX_POOL_TIMEOUT);
 				if(ret==E_TIMEDOUT)
 					{
@@ -601,7 +685,7 @@ THREAD_RETURN fm_loopReadFromMix(void* pParam)
 						if(ret!=MIXPACKET_SIZE)
 							{
 								pFirstMix->m_bRestart=true;
-								CAMsg::printMsg(LOG_ERR,"CAFirstMix::lm_loopReadFromMix - received returned: %i\n",ret);
+								CAMsg::printMsg(LOG_ERR,"CAFirstMix::lm_loopReadFromMix - received returned: %i -- restarting!\n",ret);
 								break;
 							}
 					}
@@ -623,6 +707,7 @@ THREAD_RETURN fm_loopReadFromMix(void* pParam)
 					#endif
 				#endif
 				pQueue->add(pMixPacket,sizeof(tQueueEntry));
+				getcurrentTimeMillis(keepaliveLast);
 			}
 		delete pQueueEntry;
 		#ifdef USE_POOL
@@ -661,7 +746,7 @@ THREAD_RETURN fm_loopAcceptUsers(void* param)
 			{
 				psocketgroupAccept->add(socketsIn[i]);
 			}
-#ifdef REPLAY_DETECTION //before we can start to accept users we have to nesure that we received the replay timestamps form the over mixes
+#ifdef REPLAY_DETECTION //before we can start to accept users we have to ensure that we received the replay timestamps form the over mixes
 		CAMsg::printMsg(LOG_DEBUG,"Waiting for Replay Timestamp from next mixes\n");
 		i=0;
 		while(!pFirstMix->getRestart()&&i<pFirstMix->m_u32MixCount-1)
@@ -703,6 +788,10 @@ THREAD_RETURN fm_loopAcceptUsers(void* param)
 										delete pNewMuxSocket;
 										if(ret==E_SOCKETCLOSED&&pFirstMix->getRestart()) //Hm, should we restart ??
 											goto END_THREAD;
+										else if(ret==E_SOCKET_LIMIT) // Hm no free sockets - wait some time to hope to get a free one...
+											{
+												msSleep(400);
+											}
 									}
 								else
 									{
@@ -762,6 +851,15 @@ SINT32 CAFirstMix::doUserLogin(CAMuxSocket* pNewUser,UINT8 peerIP[4])
 		#else
 			((CASocket*)pNewUser)->setKeepAlive(true);
 		#endif
+		
+		if (options.getMaxNrOfUsers() > 0 && m_nUser >= options.getMaxNrOfUsers())
+		{
+			CAMsg::printMsg(LOG_DEBUG,"Too many users: %d (Maximum:%d)\n", m_nUser, options.getMaxNrOfUsers());
+			delete pNewUser;
+			m_pIPList->removeIP(peerIP);
+			return E_UNKNOWN;
+		}
+		
 		((CASocket*)pNewUser)->send(m_xmlKeyInfoBuff,m_xmlKeyInfoSize);  // send the mix-keys to JAP
 		// es kann nicht blockieren unter der Annahme das der TCP-Sendbuffer > m_xmlKeyInfoSize ist....
 		//wait for keys from user
@@ -1339,4 +1437,4 @@ THREAD_RETURN iplist_loopDoLogCountries(void* param)
 		THREAD_RETURN_SUCCESS;	
 	}
 #endif
-
+#endif //ONLY_LOCAL_PROXY
