@@ -27,6 +27,8 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMA
 */
 #ifndef __CAFRISTMIXCHANNELLIST__
 #define __CAFRISTMIXCHANNELLIST__
+#include "doxygen.h"
+#ifndef ONLY_LOCAL_PROXY
 #include "CAMuxSocket.hpp"
 #include "CAQueue.hpp"
 #include "CASymCipher.hpp"
@@ -201,6 +203,14 @@ class CAFirstMixChannelList
 			fmChannelListEntry* getNextChannel(fmChannelListEntry* pEntry);
 		
 			static SINT32 test();
+
+      #ifdef NEW_MIX_TYPE
+        /* additional methods for TypeB first mixes */
+        SINT32 removeClientPart(CAMuxSocket* pMuxSocket);
+        void removeVacantOutChannel(fmChannelListEntry* pEntry);
+        void cleanVacantOutChannels();
+      #endif
+
 		private:
 			/** Gets the in-channel and all associated information for the given out-channel.
 				* This method is NOT thread safe (and so only for internal use)
@@ -286,3 +296,4 @@ class CAFirstMixChannelList
 #endif
 	};
 #endif
+#endif //ONLY_LOCAL_PROXY
