@@ -26,6 +26,7 @@ IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISI
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 */
 #include "StdAfx.h"
+#ifndef ONLY_LOCAL_PROXY
 #include "CAIPList.hpp"
 #include "CAMsg.hpp"
 #include "CAUtil.hpp"
@@ -209,4 +210,4 @@ SINT32 CAIPList::insertIP(const UINT8 ip[4])
 				return 0;
 			}	
 	}
-
+#endif //ONLY_LOCAL_PROXY
