@@ -27,6 +27,7 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMA
 */
 #ifndef __CAINFOSERVICE__
 #define __CAINFOSERVICE__
+#ifndef ONLY_LOCAL_PROXY
 #include "CASignature.hpp"
 #include "CAFirstMix.hpp"
 #include "CAThread.hpp"
@@ -43,12 +44,17 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMA
 
 class CAInfoService
 	{
+		#ifdef PAYMENT			
+			SINT32 getPaymentInstance(const UINT8* a_pstrPIID,CAXMLBI** pXMLBI,
+										CASocketAddrINet* a_socketAddress);
+		#endif
+								
 		public:
 			CAInfoService();
 			CAInfoService(CAMix* pMix);
 			~CAInfoService();
 			SINT32 sendMixHelo(SINT32 requestCommand=-1,const UINT8* param=NULL);
-			SINT32 sendMixInfo(const UINT8* pMixID);
+			//SINT32 sendMixInfo(const UINT8* pMixID);
 			SINT32 sendCascadeHelo();
 			SINT32 sendStatus(bool bIncludeCerts);
 			SINT32 start();
@@ -62,7 +68,8 @@ class CAInfoService
 				{
 					return m_bRun;
 				}
-			SINT32 setSignature(CASignature* pSignature,CACertificate* ownCert);
+			SINT32 setSignature(CASignature* pSignature, CACertificate* a_ownCert,
+								CACertificate** a_opCerts, UINT32 a_opCertsLength);
 
 			// added by ronin <ronin2@web.de>
 			bool isConfiguring()
@@ -75,9 +82,36 @@ class CAInfoService
 					m_bConfiguring = a_configuring;
 			}
 
+			void setSerial(UINT64 a_serial)
+			{
+				m_serial = a_serial;
+			}
+	
+			UINT8* getStatusXMLAsString(bool bIncludeCerts,UINT32& len);
+			
+#ifdef DYNAMIC_MIX
+			/** LERNGRUPPE */
+			SINT32 dynamicCascadeConfiguration();
+			bool newCascadeAvailable();
+#endif
+		public:
+			static const UINT64 MINUTE;
+			static const UINT64 SEND_LOOP_SLEEP;
+			static const UINT64 SEND_CASCADE_INFO_WAIT;
+			static const UINT64 SEND_MIX_INFO_WAIT;
+			static const UINT64 SEND_STATUS_INFO_WAIT;
+			static const UINT32 SEND_INFO_TIMEOUT_IN_SECONDS;
+			
 		private:
+			UINT8* getCascadeHeloXMLAsString(UINT32& len);
+			SINT32 sendCascadeHelo(const UINT8* xml,UINT32 len,const CASocketAddrINet* a_socketAddress) const;
+			
+			SINT32 sendStatus(const UINT8* strStatusXML,UINT32 len,const CASocketAddrINet* a_socketAddress) const;
+			UINT8* getMixHeloXMLAsString(UINT32& len);
+			SINT32 sendMixHelo(const UINT8* strMixHeloXML,UINT32 len,SINT32 requestCommand,const UINT8* param,
+								const CASocketAddrINet* a_socketAddress);
 			// added by ronin <ronin2@web.de>
-			SINT32 handleConfigEvent(DOM_Document& doc);
+			SINT32 handleConfigEvent(DOM_Document& doc) const;
 
 			volatile bool m_bRun;
 			CASignature*	m_pSignature;
@@ -85,8 +119,13 @@ class CAInfoService
 			CAMix*				m_pMix;
 			CAThread*			m_pthreadRunLoop;
 			UINT64				m_lastMixedPackets;
+			UINT64				m_serial;
 			UINT32				m_minuts;
 			SINT32				m_expectedMixRelPos;
 			bool					m_bConfiguring;
+#ifdef DYNAMIC_MIX
+			bool m_bReconfig;
+#endif
 };
 #endif
+#endif //ONLY_LOCAL_PROXY
