@@ -26,8 +26,14 @@ IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISI
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 */
 #include "StdAfx.h"
+#ifndef ONLY_LOCAL_PROXY
 #include "CALastMix.hpp"
-#include "CALastMixChannelList.hpp"
+#ifdef NEW_MIX_TYPE // TypeB mixes
+  #include "TypeB/typedefsb.hpp"
+#else // not TypeB mixes
+  /* TypeB mixes doesn't use the default-implementation */
+  #include "CALastMixChannelList.hpp"
+#endif
 #include "CASocketGroup.hpp"
 #include "CASingleSocketGroup.hpp"
 #include "CAMsg.hpp"
@@ -128,8 +134,10 @@ SINT32 CALastMix::init()
 		CAMsg::printMsg(LOG_INFO,"connected!\n");
 
 #ifdef LOG_CRIME
-		m_nCrimeRegExp=0;
-		m_pCrimeRegExps=options.getCrimeRegExps(&m_nCrimeRegExp);
+		m_nCrimeRegExpsURL=0;
+		m_pCrimeRegExpsURL=options.getCrimeRegExpsURL(&m_nCrimeRegExpsURL);
+		m_nCrimeRegExpsPayload = 0;
+		m_pCrimeRegExpsPayload = options.getCrimeRegExpsPayload(&m_nCrimeRegExpsPayload);
 #endif	
 		ret=processKeyExchange();
 		if(ret!=E_SUCCESS)
@@ -164,7 +172,10 @@ SINT32 CALastMix::init()
 		m_pLogPacketStats->setLogIntervallInMinutes(LM_PACKET_STATS_LOG_INTERVALL);
 		m_pLogPacketStats->start();
 #endif		
+    #ifndef NEW_MIX_TYPE // not TypeB mixes
+      /* TypeB mixes are using an own implementation */
 		m_pChannelList=new CALastMixChannelList;
+    #endif
 		return E_SUCCESS;
 	}
 
@@ -187,22 +198,37 @@ SINT32 CALastMix::processKeyExchange()
     //options.getCascadeName(cName,128);
     //setDOMElementAttribute(elemMixes,"cascadeName",cName);
 		doc.appendChild(elemMixes);
-		DOM_Element elemMix=doc.createElement("Mix");
-		UINT8 idBuff[50];
-		options.getMixId(idBuff,50);
-		elemMix.setAttribute("id",(char*)idBuff);
-		elemMixes.appendChild(elemMix);
+		
+		addMixInfo(elemMixes, false);
+		DOM_Element elemMix;
+		getDOMChildByName(elemMixes, (UINT8*)"Mix", elemMix, false);
 
 		//Inserting MixProtocol Version 
 		// Version 0.3  - "normal", initial mix protocol
 		// Version 0.4  - with new flow control
+    // Version 0.5  - end-to-end 1:n channels (only between client and last mix)
 		DOM_Element elemMixProtocolVersion=doc.createElement("MixProtocolVersion");
 		elemMix.appendChild(elemMixProtocolVersion);
-#ifdef NEW_FLOW_CONTROL
+    #ifdef NEW_MIX_TYPE // TypeB mixes
+      setDOMElementValue(elemMixProtocolVersion,(UINT8*)"0.5");
+      DOM_Element elemDownstreamPackets = doc.createElement("DownstreamPackets");
+      setDOMElementValue(elemDownstreamPackets, (UINT32)CHANNEL_DOWNSTREAM_PACKETS);
+      elemMixProtocolVersion.appendChild(elemDownstreamPackets);
+      DOM_Element elemChannelTimeout = doc.createElement("ChannelTimeout");
+      /* let the client use our channel-timeout + 5 seconds */
+      setDOMElementValue(elemChannelTimeout, (UINT32)(CHANNEL_TIMEOUT + 5));
+      elemMixProtocolVersion.appendChild(elemChannelTimeout);
+      DOM_Element elemChainTimeout = doc.createElement("ChainTimeout");
+      /* let the client use our chain-timeout - 5 seconds */
+      setDOMElementValue(elemChainTimeout, (UINT32)(CHAIN_TIMEOUT - 5));
+      elemMixProtocolVersion.appendChild(elemChainTimeout);
+    #else
+      #ifdef NEW_FLOW_CONTROL
 		setDOMElementValue(elemMixProtocolVersion,(UINT8*)"0.4");
-#else		
+      #else    
 		setDOMElementValue(elemMixProtocolVersion,(UINT8*)"0.3");
-#endif
+      #endif
+    #endif
 		//Inserting RSA-Key
 		DOM_DocumentFragment tmpDocFrag;
 		m_pRSA->getPublicKeyAsDocumentFragment(tmpDocFrag);
@@ -220,20 +246,30 @@ SINT32 CALastMix::processKeyExchange()
 		setDOMElementValue(elemNonce,tmpBuff);
 		elemMix.appendChild(elemNonce);
 		
-		CACertificate* ownCert=options.getOwnCertificate();
-		if(ownCert==NULL)
-			{
-				CAMsg::printMsg(LOG_DEBUG,"Own Test Cert is NULL -- so it could not be inserted into signed KeyInfo send to users...\n");
-			}	
-		CACertStore* tmpCertStore=new CACertStore();
-		tmpCertStore->add(ownCert);
-		if(m_pSignature->signXML(elemMix,tmpCertStore)!=E_SUCCESS)
-			{
-				CAMsg::printMsg(LOG_DEBUG,"Could not sign KeyInfo send to users...\n");
-			}
-		delete ownCert;
-		delete tmpCertStore;
+    
 		
+// Add Info about KeepAlive traffic
+		DOM_Element elemKeepAlive;
+		UINT32 u32KeepAliveSendInterval=options.getKeepAliveSendInterval();
+		UINT32 u32KeepAliveRecvInterval=options.getKeepAliveRecvInterval();
+		elemKeepAlive=doc.createElement("KeepAlive");
+		DOM_Element elemKeepAliveSendInterval;
+		DOM_Element elemKeepAliveRecvInterval;
+		elemKeepAliveSendInterval=doc.createElement("SendInterval");
+		elemKeepAliveRecvInterval=doc.createElement("ReceiveInterval");
+		elemKeepAlive.appendChild(elemKeepAliveSendInterval);
+		elemKeepAlive.appendChild(elemKeepAliveRecvInterval);
+		setDOMElementValue(elemKeepAliveSendInterval,u32KeepAliveSendInterval);
+		setDOMElementValue(elemKeepAliveRecvInterval,u32KeepAliveRecvInterval);
+		elemMix.appendChild(elemKeepAlive);
+		CAMsg::printMsg(LOG_DEBUG,"KeepAlive-Traffic: Offering -- SendInterval %u -- Receive Interval %u\n",u32KeepAliveSendInterval,u32KeepAliveRecvInterval);		
+		
+		// create signature
+		if (signXML(elemMix) != E_SUCCESS)
+		{
+			CAMsg::printMsg(LOG_DEBUG,"Could not sign KeyInfo sent to users...\n");
+		}
+
 		UINT32 len=0;
 		UINT8* messageBuff=DOM_Output::dumpToMem(doc,&len);
 		UINT16 tmp=htons((UINT16)len);
@@ -275,7 +311,7 @@ SINT32 CALastMix::processKeyExchange()
 		delete pCert;
 		if(oSig.verifyXML(messageBuff,len)!=E_SUCCESS)
 			{
-				CAMsg::printMsg(LOG_CRIT,"Couldt not verify the symetric key!\n");		
+				CAMsg::printMsg(LOG_CRIT,"Could not verify the symetric key!\n");		
 				delete []messageBuff;
 				return E_UNKNOWN;
 			}
@@ -299,7 +335,6 @@ SINT32 CALastMix::processKeyExchange()
 				delete []messageBuff;
 				return E_UNKNOWN;
 			}
-		
 		CAMsg::printMsg(LOG_INFO,"Verified the symetric key!\n");		
 		
 		UINT8 key[150];
@@ -317,19 +352,40 @@ SINT32 CALastMix::processKeyExchange()
 				return E_UNKNOWN;
 			}
 		m_pMuxIn->setCrypt(true);
+		///Getting and calculating the KeepAlive Traffice...
+		elemKeepAlive=NULL;
+		elemKeepAliveSendInterval=NULL;
+		elemKeepAliveRecvInterval=NULL;
+		getDOMChildByName(elemRoot,(UINT8*)"KeepAlive",elemKeepAlive);
+		getDOMChildByName(elemKeepAlive,(UINT8*)"SendInterval",elemKeepAliveSendInterval);
+		getDOMChildByName(elemKeepAlive,(UINT8*)"ReceiveInterval",elemKeepAliveRecvInterval);
+		UINT32 tmpSendInterval,tmpRecvInterval;
+		getDOMElementValue(elemKeepAliveSendInterval,tmpSendInterval,0xFFFFFFFF); //if now send interval was given set it to "infinite"
+		getDOMElementValue(elemKeepAliveRecvInterval,tmpRecvInterval,0xFFFFFFFF); //if no recv interval was given --> set it to "infinite"
+		CAMsg::printMsg(LOG_DEBUG,"KeepAlive-Traffic: Getting offer -- SendInterval %u -- Receive Interval %u\n",tmpSendInterval,tmpRecvInterval);		
+		m_u32KeepAliveSendInterval=max(u32KeepAliveSendInterval,tmpRecvInterval);
+		if(m_u32KeepAliveSendInterval>10000)
+			m_u32KeepAliveSendInterval-=10000; //make the send interval a little bit smaller than the related receive intervall
+		m_u32KeepAliveRecvInterval=max(u32KeepAliveRecvInterval,tmpSendInterval);
+		CAMsg::printMsg(LOG_DEBUG,"KeepAlive-Traffic: Calculated -- SendInterval %u -- Receive Interval %u\n",m_u32KeepAliveSendInterval,m_u32KeepAliveRecvInterval);		
 		return E_SUCCESS;
 	}
 
+#ifdef NEW_MIX_TYPE // TypeB mixes
+  void CALastMix::reconfigureMix() {
+  }
+#endif
+
 SINT32 CALastMix::reconfigure()
 	{
 		CAMsg::printMsg(LOG_DEBUG,"Reconfiguring Last Mix\n");
 		CAMsg::printMsg(LOG_DEBUG,"Re-read cache proxies\n");
 		if(setTargets()!=E_SUCCESS)
 			CAMsg::printMsg(LOG_DEBUG,"Could not set new cache proxies\n");
-#if defined (DELAY_CHANNELS)||defined (DELAY_CHANNELS_LATENCY)
+    #ifndef NEW_MIX_TYPE // not TypeB mixes
+      #if defined (DELAY_CHANNELS)||defined (DELAY_CHANNELS_LATENCY)
 		CAMsg::printMsg(LOG_DEBUG,"Set new ressources limitation parameters\n");
-		if(m_pChannelList!=NULL)
-		{
+        if(m_pChannelList!=NULL) {
 		#if defined (DELAY_CHANNELS)
 			m_pChannelList->setDelayParameters(	options.getDelayChannelUnlimitTraffic(),
 																					options.getDelayChannelBucketGrow(),
@@ -340,7 +396,10 @@ SINT32 CALastMix::reconfigure()
 			m_pChannelList->setDelayLatencyParameters(	utemp);
 		#endif
 		}
-#endif		
+      #endif
+    #else // TypeB mixes
+      reconfigureMix();
+    #endif
 		return E_SUCCESS;
 	}
 
@@ -380,6 +439,7 @@ THREAD_RETURN lm_loopSendToMix(void* param)
 		CAMuxSocket* pMuxSocket=pLastMix->m_pMuxIn;
 		SINT32 ret;
 		UINT32 len;
+		UINT32 u32KeepAliveSendInterval=pLastMix->m_u32KeepAliveSendInterval;
 #ifndef USE_POOL
 		tQueueEntry* pQueueEntry=new tQueueEntry;
 		MIXPACKET* pMixPacket=&pQueueEntry->packet;
@@ -387,8 +447,14 @@ THREAD_RETURN lm_loopSendToMix(void* param)
 		while(!pLastMix->m_bRestart)
 			{
 				len=sizeof(tQueueEntry);
-				ret=pQueue->getOrWait((UINT8*)pQueueEntry,&len);
-				if(ret!=E_SUCCESS||len!=sizeof(tQueueEntry))
+				ret=pQueue->getOrWait((UINT8*)pQueueEntry,&len,u32KeepAliveSendInterval);
+				if(ret==E_TIMEDOUT)
+					{
+						pMixPacket->flags=CHANNEL_DUMMY;
+						pMixPacket->channel=DUMMY_CHANNEL;
+						getRandom(pMixPacket->data,DATA_SIZE);
+					}
+				else if(ret!=E_SUCCESS||len!=sizeof(tQueueEntry))
 					{
 						CAMsg::printMsg(LOG_ERR,"CALastMix::lm_loopSendToMix - Error in dequeueing MixPaket\n");
 						CAMsg::printMsg(LOG_ERR,"ret=%i len=%i\n",ret,len);
@@ -454,6 +520,7 @@ THREAD_RETURN lm_loopSendToMix(void* param)
 	}
 
 #define MAX_READ_FROM_PREV_MIX_QUEUE_SIZE	10000000
+
 /* How to end this thread:
  * 1. set m_brestart=true
  */  	
@@ -469,36 +536,60 @@ THREAD_RETURN lm_loopReadFromMix(void *pParam)
 		#ifdef USE_POOL
 			CAPool* pPool=new CAPool(MIX_POOL_SIZE);
 		#endif
-		
+		UINT64 keepaliveNow,keepaliveLast;
+		UINT32 u32KeepAliveRecvInterval=pLastMix->m_u32KeepAliveRecvInterval;
+		getcurrentTimeMillis(keepaliveLast);
 		while(!pLastMix->m_bRestart)
 			{
 				if(pQueue->getSize()>MAX_READ_FROM_PREV_MIX_QUEUE_SIZE)
 					{
 						msSleep(200);
+						getcurrentTimeMillis(keepaliveLast);
 						continue;
 					}
-				SINT32 ret=pSocketGroup->select(MIX_POOL_TIMEOUT);	
-				if(ret==E_TIMEDOUT)
+				//check if the connection is broken because we did not received a Keep_alive-Message
+				getcurrentTimeMillis(keepaliveNow);
+				UINT32 keepaliveDiff=diff64(keepaliveNow,keepaliveLast);
+				if(keepaliveDiff>u32KeepAliveRecvInterval)
 					{
-						#ifdef USE_POOL
-							pMixPacket->flags=CHANNEL_DUMMY;
-							pMixPacket->channel=DUMMY_CHANNEL;
-							getRandom(pMixPacket->data,DATA_SIZE);
-							#ifdef LOG_PACKET_TIMES
-								setZero64(pQueueEntry->timestamp_proccessing_start);
-							#endif
-						#else
-							continue;	
-						#endif	
+						pLastMix->m_bRestart=true;
+						break;
+					}
+				SINT32 ret=pSocketGroup->select(MIX_POOL_TIMEOUT);	
+        if(ret < 0) 
+					{
+						if (ret == E_TIMEDOUT) 
+							{
+								#ifdef USE_POOL
+									pMixPacket->flags=CHANNEL_DUMMY;
+									pMixPacket->channel=DUMMY_CHANNEL;
+									getRandom(pMixPacket->data,DATA_SIZE);
+									#ifdef LOG_PACKET_TIMES
+										setZero64(pQueueEntry->timestamp_proccessing_start);
+									#endif
+								#else
+									continue;	
+								#endif	
+							}
+						else 
+							{
+								/* another error occured (happens sometimes while debugging because
+								 * of interruption, if a breakpoint is reached -> poll() returns
+								 * errorcode EINTR)
+								 * Note: Any Error on select() does not mean, that the underliny connections have some error state, because
+								 * in this case select() returns the socket and than this socket returns the error
+								 */
+								continue;
+							}
 					}
 				else if(ret>0)
 					{
-						ret=pMuxSocket->receive(pMixPacket); //receives a whol MixPacket
+						ret=pMuxSocket->receive(pMixPacket); //receives a whole MixPacket
 						#ifdef LOG_PACKET_TIMES
 							getcurrentTimeMicros(pQueueEntry->timestamp_proccessing_start);
 						#endif
 						if(ret!=MIXPACKET_SIZE)
-							{
+							{//something goes wrong...
 								CAMsg::printMsg(LOG_ERR,"CALastMix::lm_loopReadFromMix - received returned: %i\n",ret);
 								pLastMix->m_bRestart=true;
 								break;
@@ -513,7 +604,8 @@ THREAD_RETURN lm_loopReadFromMix(void *pParam)
 						getcurrentTimeMicros(pQueueEntry->pool_timestamp_out);
 					#endif
 				#endif		
-				pQueue->add(pQueueEntry,sizeof(tQueueEntry));	
+				pQueue->add(pQueueEntry,sizeof(tQueueEntry));
+				getcurrentTimeMillis(keepaliveLast);
 			}
 		delete pQueueEntry;
 		#ifdef USE_POOL
@@ -523,27 +615,59 @@ THREAD_RETURN lm_loopReadFromMix(void *pParam)
 	}
 
 #ifdef LOG_CRIME
-	bool CALastMix::checkCrime(UINT8* payLoad,UINT32 payLen)
-		{ //Lots of TODO!!!!
-			//DNS Lookup may block if Host does not exists!!!!!
-			//so we use regexp....
-			if(payLen<3)
-				return false;
-			UINT8* startOfUrl=(UINT8*)memchr(payLoad,32,payLen-1); //search for first space...
-			if(startOfUrl==NULL)
-				return false;
-			startOfUrl++;
-			UINT8* endOfUrl=(UINT8*)memchr(startOfUrl,32,payLen-(startOfUrl-payLoad)); //search for first space after start of URL 
-			if(endOfUrl==NULL)
-				return false;
-			UINT32 strLen=endOfUrl-startOfUrl;
-			for(UINT32 i=0;i<m_nCrimeRegExp;i++)
-				{
-					if(regnexec(&m_pCrimeRegExps[i],(char*)startOfUrl,strLen,0,NULL,0)==0)
-						return true;
-				}
-			return false;			
+	bool CALastMix::checkCrime(const UINT8* payLoad,UINT32 payLen)
+	{ 
+		//Lots of TODO!!!!
+		//DNS Lookup may block if Host does not exists!!!!!
+		//so we use regexp....
+
+		UINT8 *startOfUrl, *endOfUrl;
+		UINT32 strLen;
+		if(payLen<3)
+		{
+			return false;
 		}
+
+		if (m_nCrimeRegExpsURL > 0)
+		{
+			startOfUrl = (UINT8*)memchr(payLoad,32,payLen-1); //search for first space...
+			if(startOfUrl==NULL)
+			{
+				return false;
+			}
+			startOfUrl++;
+			//search for first space after start of URL 
+			endOfUrl = (UINT8*)memchr(startOfUrl, 32 , payLen - (startOfUrl - payLoad)); 
+			if(endOfUrl==NULL)
+			{
+				return false;
+			}
+			strLen = endOfUrl-startOfUrl;
+	
+			for(UINT32 i = 0; i < m_nCrimeRegExpsURL; i++)
+			{
+				if(regnexec(&m_pCrimeRegExpsURL[i],(char*)startOfUrl,strLen,0,NULL,0)==0)
+				{
+					return true;
+				}
+			}
+		}
+
+		if (m_nCrimeRegExpsPayload == 0)
+			{
+				// there are no regular expressions for Payload
+				return false;
+			}
+		
+		for(UINT32 i = 0; i < m_nCrimeRegExpsPayload; i++)
+		{
+			if (regnexec(&m_pCrimeRegExpsPayload[i],(const char*)payLoad ,payLen,0,NULL,0)==0)
+			{
+				return true;
+			}
+		}
+		return false;			
+	}
 #endif
 
 /** Reads the configured proxies from \c options.
@@ -628,9 +752,12 @@ SINT32 CALastMix::clean()
 		if(m_pQueueReadFromMix!=NULL)
 			delete m_pQueueReadFromMix;
 		m_pQueueReadFromMix=NULL;	
+    #ifndef NEW_MIX_TYPE // not TypeB mixes
+      /* TypeB mixes are using an own implementation */
 		if(m_pChannelList!=NULL)
 			delete m_pChannelList;
 		m_pChannelList=NULL;
+    #endif
 		return E_SUCCESS;
 	}
 
@@ -641,3 +768,4 @@ SINT32 CALastMix::initMixCascadeInfo(DOM_Element& mixes)
     setDOMElementAttribute(cascade,"create",(UINT8*)"true");
     return r;
 }
+#endif //ONLY_LOCAL_PROXY
