@@ -27,6 +27,7 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMA
 */
 #ifndef __CALASTMIXCHANNELLIST__
 #define __CALASTMIXCHANNELLIST__
+#ifndef ONLY_LOCAL_PROXY
 #include "CASocket.hpp"
 #include "CAMuxSocket.hpp"
 #include "CAQueue.hpp"
@@ -85,7 +86,9 @@ typedef struct t_lastmixchannellist lmChannelList;
 typedef struct t_lastmixchannellist lmChannelListEntry; 
 typedef lmChannelListEntry* LP_lmChannelListEntry;
 
-
+			#ifdef DELAY_CHANNELS
+				THREAD_RETURN lml_loopDelayBuckets(void*);
+	    #endif
 class CALastMixChannelList
 	{
 		public:
@@ -166,3 +169,4 @@ class CALastMixChannelList
 			#endif
 	};
 #endif
+#endif //ONLY_LOCAL_PROXY
