@@ -60,6 +60,33 @@ void SIGUSR2_handler(int signum)
 	}
 
 
+SINT32 CALocalProxy::start()
+	{
+		if(initOnce()!=E_SUCCESS)
+			return E_UNKNOWN;
+    while(true)
+    {
+				CAMsg::printMsg(LOG_DEBUG, "CALocalProxy main: before init()\n");
+        if(init() == E_SUCCESS)
+        {
+					CAMsg::printMsg(LOG_DEBUG, "CALocalProxy main: init() returned success\n");
+          CAMsg::printMsg(LOG_INFO, "The local proxy is now on-line.\n");
+					loop();
+					CAMsg::printMsg(LOG_DEBUG, "CAMix local proxy: loop() returned, maybe connection lost.\n");
+        }
+        else
+        {
+            CAMsg::printMsg(LOG_DEBUG, "init() failed, maybe no connection.\n");
+        }
+
+				CAMsg::printMsg(LOG_DEBUG, "CALocalProxy main: before clean()\n");
+				clean();
+				CAMsg::printMsg(LOG_DEBUG, "CAMix LocalProxy: after clean()\n");
+        sSleep(20);
+    }
+}
+
+
 SINT32 CALocalProxy::initOnce()
 	{
 		if(options.getListenerInterfaceCount()<1)
@@ -482,6 +509,7 @@ SINT32 CALocalProxy::clean()
 SINT32 CALocalProxy::processKeyExchange(UINT8* buff,UINT32 len)
 	{
 		CAMsg::printMsg(LOG_INFO,"Login process and key exchange started...\n");
+#ifndef ONLY_LOCAL_PROXY
 		//Parsing KeyInfo received from Mix n+1
 		MemBufInputSource oInput(buff,len,"localoproxy");
 		DOMParser oParser;
@@ -579,7 +607,25 @@ SINT32 CALocalProxy::processKeyExchange(UINT8* buff,UINT32 len)
 				return E_UNKNOWN;
 #endif
 			}
-		
+#else
+	m_MixCascadeProtocolVersion=MIX_CASCADE_PROTOCOL_VERSION_0_4;
+	m_chainlen=2;
+	m_arRSA=new CAASymCipher[m_chainlen];
+	UINT8* modulus;
+	UINT32 moduluslen;
+	UINT8* exponent;
+	UINT32 exponentlen;
+	modulus=(UINT8*)strstr((char*)buff,"<Modulus>")+9;
+	moduluslen=((UINT8*)strstr((char*)modulus,"</Modulus>"))-modulus;
+	exponent=(UINT8*)strstr((char*)modulus,"<Exponent>")+10;
+	exponentlen=((UINT8*)strstr((char*)exponent,"</Exponent>"))-exponent;
+	m_arRSA[1].setPublicKey(modulus,moduluslen,exponent,exponentlen);
+	modulus=(UINT8*)strstr((char*)exponent,"<Modulus>")+9;
+	moduluslen=((UINT8*)strstr((char*)modulus,"</Modulus>"))-modulus;
+	exponent=(UINT8*)strstr((char*)modulus,"<Exponent>")+10;
+	exponentlen=((UINT8*)strstr((char*)exponent,"</Exponent>"))-exponent;
+	m_arRSA[0].setPublicKey(modulus,moduluslen,exponent,exponentlen);
+#endif
 		//Now sending SymKeys....
 		if(m_MixCascadeProtocolVersion==MIX_CASCADE_PROTOCOL_VERSION_0_2)
 			{
@@ -599,72 +645,53 @@ SINT32 CALocalProxy::processKeyExchange(UINT8* buff,UINT32 len)
 			}
 		else
 			{
-	      DOM_Document doc=DOM_Document::createDocument();
-   			DOM_Element e = doc.createElement("JAPKeyExchange");
-				doc.appendChild(e);
-				e.setAttribute("version", "0.1");
-				DOM_Element elemLinkEnc = doc.createElement("LinkEncryption");
+				const char* XML_HEADER="<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
+				const UINT32 XML_HEADER_SIZE=strlen(XML_HEADER);
+				const char* XML_JAP_KEY_TEMPLATE="<JAPKeyExchange version=\"0.1\"><LinkEncryption>%s</LinkEncryption><MixEncryption>%s</MixEncryption></JAPKeyExchange>";
+	      //DOM_Document doc=DOM_Document::createDocument();
+   			//DOM_Element e = doc.createElement("JAPKeyExchange");
+				//doc.appendChild(e);
+				//e.setAttribute("version", "0.1");
+				//DOM_Element elemLinkEnc = doc.createElement("LinkEncryption");
+				UINT8* buff=new UINT8[9000];
 				UINT8 linkKeys[64];
 				getRandom(linkKeys,64);
-				UINT8 outBuff[512];
-				UINT32 outlen=512;
-				CABase64::encode(linkKeys,64,outBuff,&outlen);
-				outBuff[outlen]=0;
-				setDOMElementValue(elemLinkEnc,outBuff);
-				e.appendChild(elemLinkEnc);
-				DOM_Element elemMixEnc = doc.createElement("MixEncryption");
+				UINT8 outBuffLinkKey[512];
+				UINT32 outlenLinkKey=512;
+				CABase64::encode(linkKeys,64,outBuffLinkKey,&outlenLinkKey);
+				outBuffLinkKey[outlenLinkKey]=0;
+				//setDOMElementValue(elemLinkEnc,outBuff);
+				//e.appendChild(elemLinkEnc);
+				//DOM_Element elemMixEnc = doc.createElement("MixEncryption");
 				UINT8 mixKeys[32];
 				getRandom(mixKeys,32);
 				m_pSymCipher=new CASymCipher();
 				m_pSymCipher->setKey(mixKeys);
 				m_pSymCipher->setIVs(mixKeys+16);
-				outlen=512;
-				CABase64::encode(mixKeys,32,outBuff,&outlen);
-				outBuff[outlen]=0;
-				setDOMElementValue(elemMixEnc,outBuff);
-				e.appendChild(elemMixEnc);
-				encryptXMLElement(doc,&m_arRSA[m_chainlen-1]);
-				UINT32 size;
-				UINT8* buff=DOM_Output::dumpToMem(doc,&size);
-				UINT16 size2=htons(size);
+				UINT8 outBuffMixKey[512];
+				UINT32 outlenMixKey=512;
+				CABase64::encode(mixKeys,32,outBuffMixKey,&outlenMixKey);
+				outBuffMixKey[outlenMixKey]=0;
+				//setDOMElementValue(elemMixEnc,outBuff);
+				//e.appendChild(elemMixEnc);
+				sprintf((char*)buff,XML_JAP_KEY_TEMPLATE,outBuffLinkKey,outBuffMixKey);
+				UINT32 encbufflen;
+				UINT8* encbuff=encryptXMLElement(buff,strlen((char*)buff),encbufflen,&m_arRSA[m_chainlen-1]);
+				UINT16 size2=htons(encbufflen+XML_HEADER_SIZE);
 				SINT32 ret=((CASocket*)&m_muxOut)->send((UINT8*)&size2,2);
-				ret=((CASocket*)&m_muxOut)->send(buff,size);
+				ret=((CASocket*)&m_muxOut)->send((UINT8*)XML_HEADER,XML_HEADER_SIZE);
+				ret=((CASocket*)&m_muxOut)->send(encbuff,encbufflen);
+				delete[] encbuff;
 				delete[] buff;
 				// Checking Signature send from Mix
 				ret=((CASocket*)&m_muxOut)->receiveFully((UINT8*)&size2,2);
 				size2=ntohs(size2);
-				/*byte[] tmpBuff = new byte[xml_buff.length + 2];
-				System.arraycopy(xml_buff, 0, tmpBuff, 2, xml_buff.length);
-				tmpBuff[0] = (byte) ( (xml_buff.length >> 8) & 0x00FF);
-				tmpBuff[1] = (byte) (xml_buff.length & 0x00FF);
-				int len = m_inDataStream.readShort();
-				byte[] mixSigBuff = new byte[len];
-				m_inDataStream.readFully(mixSigBuff);*/
-				buff=new UINT8[size2];
-				ret=((CASocket*)&m_muxOut)->receiveFully(buff,size2);
-				delete[] buff;
+				UINT8* xmlbuff=new UINT8[size2];
+				ret=((CASocket*)&m_muxOut)->receiveFully(xmlbuff,size2);
+				delete[] xmlbuff;
 				m_muxOut.setSendKey(linkKeys,32);
 				m_muxOut.setReceiveKey(linkKeys+32,32);
 				m_muxOut.setCrypt(true);
-				/*doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(new
-					ByteArrayInputStream(mixSigBuff));
-				root = doc.getDocumentElement();
-				if (!root.getNodeName().equals("Signature"))
-				{
-					return ErrorCodes.E_UNKNOWN;
-				}
-				Node elemSigValue = XMLUtil.getFirstChildByName(root, "SignatureValue");
-				String strSigValue = XMLUtil.parseValue(elemSigValue, null);
-				byte[] sigValue = Base64.decode(strSigValue);
-				JAPCertificate certs[] = JAPSignature.getAppendedCertificates(nodeSig);
-				JAPSignature sig = new JAPSignature();
-				sig.initVerify(certs[0].getPublicKey());
-				if (!sig.verify(tmpBuff, sigValue, true))
-				{
-					return ErrorCodes.E_UNKNOWN;
-				}
-				setEnableEncryption(true);
-				*/
 			}
 		CAMsg::printMsg(LOG_INFO,"Login process and key exchange finished!\n");		
 		return E_SUCCESS;
