@@ -26,6 +26,7 @@ IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISI
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 */
 #include "StdAfx.h"
+#ifndef ONLY_LOCAL_PROXY
 #include "CAMiddleMix.hpp"
 #include "CASingleSocketGroup.hpp"
 #include "CAMsg.hpp"
@@ -201,7 +202,11 @@ SINT32 CAMiddleMix::processKeyExchange()
 		count++;
 		setDOMElementAttribute(root,"count",count);
 		
-		DOM_Element mixNode=doc.createElement(DOMString("Mix"));
+		addMixInfo(root, true);
+		DOM_Element mixNode;
+		getDOMChildByName(root, (UINT8*)"Mix", mixNode, false);
+		
+		
 		UINT8 tmpBuff[50];
 		options.getMixId(tmpBuff,50); //the mix id...
 		mixNode.setAttribute("id",DOMString((char*)tmpBuff));
@@ -224,21 +229,13 @@ SINT32 CAMiddleMix::processKeyExchange()
 		setDOMElementValue(elemNonce,tmpBuff);
 		mixNode.appendChild(elemNonce);
 		
-		CACertificate* ownCert=options.getOwnCertificate();
-		if(ownCert==NULL)
-			{
-				CAMsg::printMsg(LOG_DEBUG,"Own Test Cert is NULL -- so it could not be inserted into signed KeyInfo send to users...\n");
-			}	
-		CACertStore* tmpCertStore=new CACertStore();
-		tmpCertStore->add(ownCert);
-		if(m_pSignature->signXML(mixNode,tmpCertStore)!=E_SUCCESS)
+		// create signature
+		if(signXML(mixNode)!=E_SUCCESS)
 			{
 				CAMsg::printMsg(LOG_DEBUG,"Could not sign KeyInfo send to users...\n");
 			}
-		delete ownCert;
-		delete tmpCertStore;
 		
-		root.insertBefore(mixNode,root.getFirstChild());
+		
 
 		UINT8* out=new UINT8[0xFFFF];
 		UINT32 outlen=0xFFFD;
@@ -288,7 +285,7 @@ SINT32 CAMiddleMix::processKeyExchange()
 		delete pCert;
 		if(oSig.verifyXML(elemRoot)!=E_SUCCESS)
 			{
-				CAMsg::printMsg(LOG_CRIT,"Couldt not verify the symetric key form Mix n-1!\n");		
+				CAMsg::printMsg(LOG_CRIT,"Could not verify the symetric key form Mix n-1!\n");		
 				return E_UNKNOWN;
 			}
 		//Verifying nonce
@@ -302,10 +299,10 @@ SINT32 CAMiddleMix::processKeyExchange()
 			memcmp(SHA1(arNonce,16,NULL),tmpBuff,SHA_DIGEST_LENGTH)!=0
 			)
 			{
-				CAMsg::printMsg(LOG_CRIT,"Couldt not verify the Nonce!\n");		
+				CAMsg::printMsg(LOG_CRIT,"Could not verify the Nonce!\n");		
 				return E_UNKNOWN;
 			}
-		CAMsg::printMsg(LOG_INFO,"Verified the symetric key!\n");		
+		CAMsg::printMsg(LOG_INFO,"Verified the symmetric key!\n");		
 
 		UINT8 key[150];
 		UINT32 keySize=150;
@@ -313,7 +310,7 @@ SINT32 CAMiddleMix::processKeyExchange()
 		ret=decodeXMLEncryptedKey(key,&keySize,elemRoot,m_pRSA);
 		if(ret!=E_SUCCESS||keySize!=64)
 			{
-				CAMsg::printMsg(LOG_CRIT,"Couldt not set the symetric key to be used by the MuxSocket!\n");		
+				CAMsg::printMsg(LOG_CRIT,"Could not set the symmetric key to be used by the MuxSocket!\n");		
 				return E_UNKNOWN;
 			}
 		m_pMuxIn->setReceiveKey(key,32);
@@ -323,6 +320,10 @@ SINT32 CAMiddleMix::processKeyExchange()
 
 SINT32 CAMiddleMix::init()
 	{		
+#ifdef DYNAMIC_MIX
+		m_bBreakNeeded = m_bReconfigured;
+#endif
+
 		CAMsg::printMsg(LOG_INFO,"Creating Key...\n");
 		m_pRSA=new CAASymCipher();
 		if(m_pRSA->generateKeyPair(1024)!=E_SUCCESS)
@@ -360,22 +361,6 @@ SINT32 CAMiddleMix::init()
 		((CASocket*)*m_pMuxOut)->setRecvBuff(50*MIXPACKET_SIZE);
 		((CASocket*)*m_pMuxOut)->setSendBuff(50*MIXPACKET_SIZE);
 
-#define RETRIES 100
-#define RETRYTIME 30
-		CAMsg::printMsg(LOG_INFO,"Init: Try to connect to next Mix...\n");
-		if(m_pMuxOut->connect(*pAddrNext,RETRIES,RETRYTIME)!=E_SUCCESS)
-			{
-				CAMsg::printMsg(LOG_CRIT,"Cannot connect to next Mix -- Exiting!\n");
-				return E_UNKNOWN;
-			}
-//		mSocketGroup.add(muxOut);
-		CAMsg::printMsg(LOG_INFO," connected!\n");
-		if(((CASocket*)*m_pMuxOut)->setKeepAlive((UINT32)1800)!=E_SUCCESS)
-			{
-				CAMsg::printMsg(LOG_INFO,"Socket option TCP-KEEP-ALIVE returned an error - so not set!\n");
-				if(((CASocket*)*m_pMuxOut)->setKeepAlive(true)!=E_SUCCESS)
-					CAMsg::printMsg(LOG_INFO,"Socket option KEEP-ALIVE returned an error - so also not set!\n");
-			}
 		
     CAMsg::printMsg(LOG_INFO,"Waiting for Connection from previous Mix...\n");    
 		CAListenerInterface* pListener=NULL;
@@ -398,6 +383,13 @@ SINT32 CAMiddleMix::init()
 		pAddr=pListener->getAddr();
 		delete pListener;
 		m_pMuxIn=new CAMuxSocket();
+#ifdef DYNAMIC_MIX
+		// LERNGRUPPE Do not block if we are currently reconfiguring
+		if(m_bBreakNeeded != m_bReconfigured)
+		{
+			return E_UNKNOWN;
+		}
+#endif
 		SINT32 ret=m_pMuxIn->accept(*pAddr);
 		delete pAddr;
 		if(ret!=E_SUCCESS)
@@ -415,6 +407,24 @@ SINT32 CAMiddleMix::init()
 					CAMsg::printMsg(LOG_INFO,"Socket option KEEP-ALIVE returned an error - so also not set!\n");
 			}
 		
+		/** Connect to next mix */
+		if(connectToNextMix(pAddrNext) != E_SUCCESS)
+		{
+			delete pAddrNext;
+			CAMsg::printMsg(LOG_DEBUG, "CAMiddleMix::init - Unable to connect to next mix\n");
+			return E_UNKNOWN;
+		}
+		delete pAddrNext;
+
+//		mSocketGroup.add(muxOut);
+		CAMsg::printMsg(LOG_INFO," connected!\n");
+		if(((CASocket*)*m_pMuxOut)->setKeepAlive((UINT32)1800)!=E_SUCCESS)
+			{
+				CAMsg::printMsg(LOG_INFO,"Socket option TCP-KEEP-ALIVE returned an error - so not set!\n");
+				if(((CASocket*)*m_pMuxOut)->setKeepAlive(true)!=E_SUCCESS)
+					CAMsg::printMsg(LOG_INFO,"Socket option KEEP-ALIVE returned an error - so also not set!\n");
+			}
+
     if((ret = processKeyExchange())!=E_SUCCESS)
 			{
 				CAMsg::printMsg(LOG_CRIT,"Error in proccessKeyExchange()!\n");
@@ -427,10 +437,9 @@ SINT32 CAMiddleMix::init()
 		return E_SUCCESS;
 	}
 
-	
+/** Processes Downstream-MixPackets.*/	
 THREAD_RETURN mm_loopDownStream(void *p)
 	{
-#ifndef NEW_MIX_TYPE
 		CAMiddleMix* pMix=(CAMiddleMix*)p;
 		HCHANNEL channelIn;
 		CASymCipher* pCipher;
@@ -507,6 +516,10 @@ THREAD_RETURN mm_loopDownStream(void *p)
 								#ifdef USE_POOL
 									pPool->pool(pPoolEntry);
 								#endif
+								if((pMixPacket->flags&CHANNEL_CLOSE)!=0)
+									{//Channel close received -->remove channel form channellist
+										pMix->m_pMiddleMixChannelList->remove(channelIn);
+									}
 								if(pMix->m_pMuxIn->send(pMixPacket)==SOCKET_ERROR)
 									goto ERR;
 							}
@@ -522,14 +535,51 @@ ERR:
 		pMix->m_pMuxIn->close();
 		pMix->m_pMuxOut->close();
 		CAMsg::printMsg(LOG_CRIT,"loopDownStream -- Now Exiting!\n");
-#endif //!NEW_MIX_TYPE
 		THREAD_RETURN_SUCCESS;		
 	}
 
+SINT32 CAMiddleMix::connectToNextMix(CASocketAddr* a_pAddrNext)
+{
+#define RETRIES 100
+#define RETRYTIME 30
+		CAMsg::printMsg(LOG_INFO,"Init: Try to connect to next Mix...\n");
+		UINT32 i = 0;
+		SINT32 err = E_UNKNOWN;
+		for(i=0; i < RETRIES; i++)
+		{
+#ifdef DYNAMIC_MIX
+			/** If someone reconfigured the mix, we need to break the loop as ne next mix might have changed */
+			if(m_bBreakNeeded != m_bReconfigured)
+			{
+				CAMsg::printMsg(LOG_DEBUG, "CAMiddleMix::connectToNextMix - Broken the connect loop!\n");
+				break;
+			}
+#endif
+			err = m_pMuxOut->connect(*a_pAddrNext);
+			if(err != E_SUCCESS)
+			{
+				err=GET_NET_ERROR;
+#ifdef _DEBUG
+			 	CAMsg::printMsg(LOG_DEBUG,"Con-Error: %i\n",err);
+#endif
+				if(err!=ERR_INTERN_TIMEDOUT&&err!=ERR_INTERN_CONNREFUSED)
+					break;
+#ifdef _DEBUG
+				CAMsg::printMsg(LOG_DEBUG,"Cannot connect... retrying\n");
+#endif				
+				sSleep(RETRYTIME);
+			}
+			else
+			{
+				break;
+			}
+		}
+		return err;
+}
 
+/** Processes Upstream-MixPackets.*/
 SINT32 CAMiddleMix::loop()
 	{
-#ifndef NEW_MIX_TYPE
 		tPoolEntry* pPoolEntry=new tPoolEntry;
 		MIXPACKET* pMixPacket=&pPoolEntry->packet;
 		HCHANNEL channelOut;
@@ -601,7 +651,7 @@ SINT32 CAMiddleMix::loop()
 						#endif
 						if(m_pMiddleMixChannelList->getInToOut(pMixPacket->channel,&channelOut,&pCipher)!=E_SUCCESS)
 							{//new connection ?
-								if(pMixPacket->flags==CHANNEL_OPEN)
+								if(pMixPacket->flags==CHANNEL_OPEN) //if not channel-open flag set -->drop this packet
 									{
 										#ifdef _DEBUG
 												CAMsg::printMsg(LOG_DEBUG,"New Connection from previous Mix!\n");
@@ -632,12 +682,16 @@ SINT32 CAMiddleMix::loop()
 							}
 						else
 							{//established connection
-									pMixPacket->channel=channelOut;
 									pCipher->crypt1(pMixPacket->data,pMixPacket->data,DATA_SIZE);
 									pCipher->unlock();
 									#ifdef USE_POOL
 										pPool->pool(pPoolEntry);
 									#endif
+									if((pMixPacket->flags&CHANNEL_CLOSE)!=0)
+										{//Channel close received -->remove channel form channellist
+											m_pMiddleMixChannelList->remove(pMixPacket->channel);
+										}
+									pMixPacket->channel=channelOut;
 									if(m_pMuxOut->send(pMixPacket)==SOCKET_ERROR)
 										goto ERR;
 							}
@@ -655,7 +709,6 @@ ERR:
 		#ifdef USE_POOL
 			delete pPool;
 		#endif
-#endif //!NEW_MIX_TYPE
 		CAMsg::printMsg(LOG_CRIT,"loop(): Seams that we are restarting now!!\n");
 		return E_UNKNOWN;
 	}
@@ -689,3 +742,4 @@ SINT32 CAMiddleMix::clean()
 		m_pMiddleMixChannelList=NULL;
 		return E_SUCCESS;
 	}
+#endif //ONLY_LOCAL_PROXY
