@@ -49,6 +49,9 @@ class CAMiddleMix:public
 					m_pMiddleMixChannelList=NULL;
 					m_pMuxOut=NULL;m_pMuxIn=NULL;m_pRSA=NULL;m_pSignature=NULL;
 					m_pInfoService=NULL;
+#ifdef DYNAMIC_MIX
+					m_bBreakNeeded = false;
+#endif
 				}
 			virtual ~CAMiddleMix(){clean();};
 			tMixType getType() const
@@ -61,13 +64,17 @@ class CAMiddleMix:public
 			SINT32 init();
 			SINT32 initOnce();
 			SINT32 clean();
+			SINT32 connectToNextMix(CASocketAddr* a_pAddrNext);
+#ifdef DYNAMIC_MIX
+			bool m_bBreakNeeded;
+#endif
 			
     /**
     * This method is not applicable to middle mixes; it does nothing
     * @param d ignored
     * @retval E_SUCCESS in any case
     */
-    virtual SINT32 initMixCascadeInfo(DOM_Element& )
+    virtual SINT32 initMixCascadeInfo(DOM_Element& d)
     {
         return E_SUCCESS;
     }
@@ -83,6 +90,15 @@ private:
 			CAMiddleMixChannelList* m_pMiddleMixChannelList;
 			//CAInfoService* m_pInfoService;
 			friend THREAD_RETURN mm_loopDownStream(void *p);
+#ifdef DYNAMIC_MIX
+protected:
+			void stopCascade()
+			{
+				m_bRun = false;
+			}
+#endif
 	};
 
+
+
 #endif
