@@ -28,6 +28,7 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMA
 #ifndef __CAMIX__
 #define __CAMIX__
 
+#ifndef ONLY_LOCAL_PROXY
 extern class CACmdLnOptions options;
 
 class CASignature;
@@ -56,6 +57,13 @@ class CAMix
 			SINT32 start();
 			virtual SINT32 reconfigure(){return E_SUCCESS;}
 			virtual tMixType getType() const =0;
+#ifdef DYNAMIC_MIX
+			void setReconfiguring(bool a_val)
+			{
+				m_bReconfiguring = a_val;
+			}
+			SINT32 dynaReconfigure(bool a_bChangeMixType);
+#endif
 
 			/** Returns the Mix-Cascade info which should be send to the InfoService.
     		* This is NOT a copy!
@@ -92,21 +100,37 @@ class CAMix
 					return m_pMuxOutControlChannelDispatcher;
 				}
 		protected:
+#ifdef DYNAMIC_MIX
+			virtual void stopCascade() =0;
+			bool m_bLoop;
+			bool m_bCascadeEstablished;
+			bool m_bReconfigured;
+#endif
+			bool m_bReconfiguring;
 			virtual SINT32 clean()=0;
 			virtual SINT32 initOnce(){return E_SUCCESS;}
 			virtual SINT32 init()=0;
 			virtual SINT32 loop()=0;
 
+			SINT32 addMixInfo(DOM_Node& a_element, bool a_bForceFirstNode);
+		
+
 			// added by ronin <ronin2@web.de>
 			virtual SINT32 processKeyExchange()
 			{
 					return E_SUCCESS;
 			}
+			
 			// added by ronin <ronin2@web.de>
 			virtual SINT32 initMixCascadeInfo(DOM_Element& elemMixes);
 
+			SINT32 signXML(DOM_Node& a_element);
+
 			CASignature* m_pSignature;
 			CAInfoService* m_pInfoService;
+			
+			UINT32 m_u32KeepAliveRecvInterval;
+			UINT32 m_u32KeepAliveSendInterval;
 
 	    bool m_acceptReconfiguration;
 
@@ -122,3 +146,4 @@ class CAMix
 			bool needAutoConfig();
 	};
 #endif
+#endif //ONLY_LOCAL_PROXY
