@@ -26,6 +26,7 @@ IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISI
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 */
 #include "StdAfx.h"
+#ifndef ONLY_LOCAL_PROXY
 #include "CASignature.hpp"
 #include "CABase64.hpp"
 #include "CAUtil.hpp"
@@ -135,7 +136,7 @@ SINT32 CASignature::setSignKey(const UINT8* buff,UINT32 len,UINT32 type,const ch
 					return parseSignKeyXML(buff,len);
 
 				case SIGKEY_PKCS12:
-					#if OPENSSL_VERSION_NUMBER	> 0x0090705fL
+					#if OPENSSL_VERSION_NUMBER	> 0x009070CfL
 						PKCS12* tmpPKCS12=d2i_PKCS12(NULL,(const UINT8**)&buff,len);	
 					#else
 						PKCS12* tmpPKCS12=d2i_PKCS12(NULL,(UINT8**)&buff,len);	
@@ -413,12 +414,23 @@ SINT32 CASignature::getVerifyKey(CACertificate** ppCert)
 		EVP_PKEY_set1_DSA(pPKey,m_pDSA);
 		*ppCert=new CACertificate();
 		(*ppCert)->m_pCert=X509_new();
-		X509_set_version((*ppCert)->m_pCert,2);
+    // LERNGRUPPE 
+    // We nned to use Version 3 to use extensions
+// 		X509_set_version((*ppCert)->m_pCert,2);
+ 		X509_set_version((*ppCert)->m_pCert,3);
 		ASN1_TIME* pTime=ASN1_TIME_new();
 		ASN1_TIME_set(pTime,time(NULL));
 		X509_set_notBefore((*ppCert)->m_pCert,pTime);
 		X509_set_notAfter((*ppCert)->m_pCert,pTime);
 		X509_set_pubkey((*ppCert)->m_pCert,pPKey);
+// LERNGRUPPE 
+// Add the subjectKeyIdentifier-Extension to the certificate
+                if( (*ppCert)->setSubjectKeyIdentifier() != E_SUCCESS )
+                {
+                    CAMsg::printMsg( LOG_ERR, "Couldn't add the SKI to the certificate!\n");
+                    return E_UNKNOWN;
+                }
+
 		X509_sign((*ppCert)->m_pCert,pPKey,EVP_sha1());
 		EVP_PKEY_free(pPKey);
 		return E_SUCCESS;
@@ -663,7 +675,7 @@ SINT32 CASignature::verifyXML(const UINT8* const in,UINT32 inlen)
 	}
 
 
-/** Verifies a XML Signature under node root. @Note This removes the \<Signature\> node! */
+/** Verifies a XML Signature under node root.*/
 SINT32 CASignature::verifyXML(DOM_Node& root,CACertStore* trustedCerts)
 	{
 		DOM_Element elemSignature;
@@ -747,10 +759,10 @@ SINT32 CASignature::verifyXML(DOM_Node& root,CACertStore* trustedCerts)
 				return E_UNKNOWN;
 			}
 		DSA_SIG_free(dsaSig);
-				
-		root.removeChild(elemSignature);
+		DOM_Node tmpNode=root.removeChild(elemSignature);
 		outlen=5000;
 		DOM_Output::makeCanonical(root,out,&outlen);
+		root.appendChild(tmpNode);
 		UINT8 dgst1[SHA_DIGEST_LENGTH];
 		SHA1(out,outlen,dgst1);
 		delete[] out;
@@ -785,3 +797,7 @@ SINT32 CASignature::decodeRS(const UINT8* in, const UINT32 inLen, DSA_SIG* pDsaS
 	return E_SUCCESS;
 }
 
+
+
+
+#endif //ONLY_LOCAL_PROXY
