@@ -41,34 +41,50 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMA
 #define CLOSE_RECEIVE 0x02
 #define CLOSE_BOTH		0x03
 
-CASocket::CASocket()
+UINT32 CASocket::m_u32NormalSocketsOpen=0; //how many "normal" sockets are open
+UINT32 CASocket::m_u32MaxNormalSockets=0xFFFFFFFF; //how many "normal" sockets are allowed at max
+
+CASocket::CASocket(bool bIsReservedSocket)
 	{
 		m_Socket=0;
 		m_closeMode=0;
 		m_bSocketIsClosed=true;
+		m_bIsReservedSocket=bIsReservedSocket;
 	}
 
 SINT32 CASocket::create()
 	{
 		return create(AF_INET);
 	}
-
+///@todo Not thread safe!
 SINT32 CASocket::create(int type)
 	{
 		if(m_bSocketIsClosed)
+			{
+			if(m_bIsReservedSocket||m_u32NormalSocketsOpen<m_u32MaxNormalSockets)
 			m_Socket=socket(type,SOCK_STREAM,0);
+			else
+				{
+				CAMsg::printMsg(LOG_CRIT,"Could not create a new normal Socket -- allowed number of normal sockets exeded!\n");
+				return SOCKET_ERROR;
+				}
+			}
 		else
 			return E_UNKNOWN;
 		if(m_Socket==INVALID_SOCKET)
 			{
 				int er=GET_NET_ERROR;
 				if(er==EMFILE)
-					CAMsg::printMsg(LOG_CRIT,"Couldt not create a new Socket!\n");
+					CAMsg::printMsg(LOG_CRIT,"Could not create a new Socket!\n");
 				else
-					CAMsg::printMsg(LOG_CRIT,"Couldt not create a new Socket! - Error: %i\n",er);
+					CAMsg::printMsg(LOG_CRIT,"Could not create a new Socket! - Error: %i\n",er);
 				return SOCKET_ERROR;
 			}
 		m_bSocketIsClosed=false;
+		m_csClose.lock();
+		if(!m_bIsReservedSocket)
+		m_u32NormalSocketsOpen++;
+		m_csClose.unlock();
 		return E_SUCCESS;
 	}
 
@@ -116,12 +132,23 @@ SINT32 CASocket::listen(UINT16 port)
 		return listen(oSocketAddrINet);
 	}
 
+/** Accepts a new connection. The new socket is returned in s.
+  * @retval E_SUCCESS if successful
+	* @retval E_SOCKETCLOSED if the listening socket was closed
+	* @retval E_SOCKET_LIMIT if the could not create a new socket for the new connection
+	* @retval E_UNKNOWN otherwise
+	*/
 SINT32 CASocket::accept(CASocket &s)
 	{
 		if(m_bSocketIsClosed) //the accept socket should not be closed!!
 			return E_SOCKETCLOSED;
 		if(!s.m_bSocketIsClosed) //but the new socket should be closed!!!
 			return E_UNKNOWN;
+		if(m_u32NormalSocketsOpen>=m_u32MaxNormalSockets)
+			{
+				CAMsg::printMsg(LOG_CRIT,"CASocket::accept() -- Could not create a new normal Socket -- allowed number of normal sockets exeded!\n");
+				return E_SOCKET_LIMIT;
+			}
 		s.m_Socket=::accept(m_Socket,NULL,NULL);
 		if(s.m_Socket==SOCKET_ERROR)
 			{
@@ -130,10 +157,9 @@ SINT32 CASocket::accept(CASocket &s)
 					return E_SOCKETCLOSED;
 				return E_UNKNOWN;
 			}
-
-#ifdef _DEBUG
-		sockets++;
-#endif
+		m_csClose.lock();
+		m_u32NormalSocketsOpen++;
+		m_csClose.unlock();
 		s.m_bSocketIsClosed=false;
 		return E_SUCCESS;
 	}
@@ -144,7 +170,7 @@ SINT32 CASocket::accept(CASocket &s)
 	* @param retry - number of retries
 	* @param time - time between retries in seconds
 	*/			
-SINT32 CASocket::connect(CASocketAddr & psa,UINT32 retry,UINT32 time)
+SINT32 CASocket::connect(const CASocketAddr & psa,UINT32 retry,UINT32 time)
 	{
 //		CAMsg::printMsg(LOG_DEBUG,"Socket:connect\n");
 		if(m_bSocketIsClosed&&create()!=E_SUCCESS)
@@ -187,7 +213,7 @@ SINT32 CASocket::connect(CASocketAddr & psa,UINT32 retry,UINT32 time)
 	*	@param psa - peer
 	* @param msTimeOut - abort after msTimeOut milli seconds
 	*/
-SINT32 CASocket::connect(CASocketAddr & psa,UINT32 msTimeOut)
+SINT32 CASocket::connect(const CASocketAddr & psa,UINT32 msTimeOut)
 	{
 		if(m_bSocketIsClosed&&create(psa.getType())!=E_SUCCESS)
 			{
@@ -256,15 +282,9 @@ SINT32 CASocket::close()
 		int ret;
 		if(!m_bSocketIsClosed)
 			{
-#ifdef _DEBUG				
-				if(::closesocket(m_Socket)==SOCKET_ERROR)
-					{
-						CAMsg::printMsg(LOG_DEBUG,"Fehler bei CASocket::closesocket\n -- %i",GET_NET_ERROR);
-					}
-				sockets--;
-#else
 				::closesocket(m_Socket);
-#endif
+			if(!m_bIsReservedSocket)
+			m_u32NormalSocketsOpen--;
 				m_bSocketIsClosed=true;
 				m_closeMode=0;
 				ret=E_SUCCESS;
@@ -274,30 +294,6 @@ SINT32 CASocket::close()
 		m_csClose.unlock();
 		return ret;
 	}
-
-/* /// it seems that this function is never used (Bastian)
-SINT32 CASocket::close(UINT32 mode)
-	{
-//		EnterCriticalSection(&csClose);
-		::shutdown(m_Socket,mode);
-		if(mode==SD_RECEIVE||mode==SD_BOTH)
-			m_closeMode|=CLOSE_RECEIVE;
-		if(mode==SD_SEND||mode==SD_BOTH)
-			{
-				m_closeMode|=CLOSE_SEND;				
-			}
-		int ret;
-		if(m_closeMode==CLOSE_BOTH)
-			{
-				close();
-				ret=E_SUCCESS;
-			}
-		else
-			ret=1;
-//		LeaveCriticalSection(&csClose);
-		return ret;
-	}
-*/
 			
 /** Sends some data over the network. This may block, 
 	* if socket is in blocking mode.
@@ -528,29 +524,6 @@ SINT32 CASocket::setReuseAddr(bool b)
 		return setsockopt(m_Socket,SOL_SOCKET,SO_REUSEADDR,(char*)&val,sizeof(val));
 	}
 
-/*
-SINT32 CASocket::setRecvLowWat(UINT32 r)
-	{
-		int val=r;
-		return setsockopt(m_Socket,SOL_SOCKET,SO_RCVLOWAT,(char*)&val,sizeof(val));
-	}
-
-SINT32 CASocket::setSendLowWat(UINT32 r)
-	{
-		int val=r;
-		return setsockopt(m_Socket,SOL_SOCKET,SO_SNDLOWAT,(char*)&val,sizeof(val));
-	}
-
-SINT32 CASocket::getSendLowWat()
-	{
-		int val;
-		socklen_t size=sizeof(val);
-		if(getsockopt(m_Socket,SOL_SOCKET,SO_SNDLOWAT,(char*)&val,&size)==SOCKET_ERROR)
-			return E_UNKNOWN;
-		else
-			return val;
-	}
-*/
 SINT32 CASocket::setRecvBuff(UINT32 r)
 	{
 		int val=r;
@@ -675,3 +648,42 @@ SINT32 CASocket::getNonBlocking(bool* b)
 				return SOCKET_ERROR;
 		#endif
 	}
+
+SINT32 CASocket::getMaxOpenSockets()
+{
+	CASocket* parSocket=new CASocket[10001];
+	UINT32 maxSocket=0;
+	for(UINT32 t=0;t<10001;t++)
+		{
+		if(parSocket[t].create()!=E_SUCCESS)
+			{
+			maxSocket=t;
+			break;
+			}
+		}
+	for(UINT32 t=0;t<maxSocket;t++)
+		{
+		parSocket[t].close();
+		}
+	delete []parSocket;
+	return maxSocket;
+}
+
+/**
+ * LERNGRUPPE
+ * Returns the source address of the socket
+ * @return r_Ip the source IP address
+ * @retval E_SUCCESS upon success
+ * @retval SOCKET_ERROR otherwise
+ *
+ * @todo: Question: Correct for Unix domain sockets?
+ */
+SINT32 CASocket::getLocalIP(UINT32* r_Ip)
+{
+      struct sockaddr_in addr;
+      socklen_t namelen=sizeof(struct sockaddr_in);
+      if(getsockname(m_Socket,(struct sockaddr*)&addr,&namelen)==SOCKET_ERROR)
+              return SOCKET_ERROR;
+      *r_Ip =  addr.sin_addr.s_addr;
+      return E_SUCCESS;
+}
