@@ -26,6 +26,7 @@ IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISI
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 */
 #include "StdAfx.h"
+#ifndef ONLY_LOCAL_PROXY
 #ifdef PAYMENT
 #include "CATLSClientSocket.hpp"
 #include "CAUtil.hpp"
@@ -208,3 +209,4 @@ SINT32 CATLSClientSocket::receive(UINT8* buff,UINT32 len)
 	}
 
 #endif
+#endif //ONLY_LOCAL_PROXY
