@@ -32,6 +32,7 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMA
 	#include "CAMsg.hpp"
 #endif
 #include "xml/DOM_Output.hpp"
+#include "CASymCipher.hpp"
 /**
 *	Removes leading and ending whitespaces (chars<=32) from a zero terminated string.
 *		@param s input string (null terminated)
@@ -145,14 +146,13 @@ char* strins(const char* src,const char * pos,const char* ins)
 			return NULL;
 		return strins(src,pos-src,ins);
 	}
-
-/** Gets the current Systemtime in milli seconds. 
+/*
+// Gets the current Systemtime in milli seconds. 
 	* @param bnTime - Big Number, in which the current time is placed
 	* @retval E_UNSPECIFIED, if bnTime was NULL
 	* @retval E_UNKNOWN, if an error occurs
 	*	@retval	E_SUCCESS, otherwise
-*/
-/*SINT32 getcurrentTimeMillis(BIGNUM* bnTime)
+SINT32 getcurrentTimeMillis(BIGNUM* bnTime)
 	{
 		if(bnTime==NULL)
 			return E_UNSPECIFIED;
@@ -356,6 +356,7 @@ SINT32 filelength(int handle)
 	}
 #endif
 
+#ifndef ONLY_LOCAL_PROXY
 SINT32 setDOMElementValue(DOM_Element& elem,UINT32 text)
 	{
 		UINT8 tmp[10];
@@ -499,6 +500,26 @@ SINT32 getDOMChildByName(const DOM_Node& node,const UINT8* const name,DOM_Node&
 			}
 		return E_UNKNOWN;
 	}
+	
+SINT32 getLastDOMChildByName(const DOM_Node& node,const UINT8* const name,DOM_Node& a_child)
+	{
+		if(node==NULL)
+			{
+				return E_UNKNOWN;
+			}
+		DOM_Node child;
+		child=node.getLastChild();
+		while(child!=NULL)
+			{
+				if(child.getNodeName().equals((char*)name))
+					{
+						a_child = child; // found a child
+						return E_SUCCESS;
+					}
+				child=child.getPreviousSibling();
+			}
+		return E_UNKNOWN;
+	}
 
 /** 
  * Returns the content of the text node(s) under elem
@@ -506,10 +527,12 @@ SINT32 getDOMChildByName(const DOM_Node& node,const UINT8* const name,DOM_Node&
  * len is set to 0.
  * 
  * TODO: Why is elem a DOM_Node and not a DOM_Element here?
- * @param DOM_Node the element which has a text node under it
+ * @param elem the element which has a text node under it
  * @param value a buffer that gets the text value
- * @param len on call contains the buffer size, on return contains the number of bytes copied
- * @return E_SPACE if the buffer is too small, E_SUCCESS otherwise, E_UNKNOWN if the element is NULL
+ * @param valuelen on call contains the buffer size, on return contains the number of bytes copied
+ * @retval E_SPACE if the buffer is too small
+ * @retval E_UNKNOWN if the element is NULL
+ * @retval E_SUCCESS otherwise
  */
 SINT32 getDOMElementValue(const DOM_Node& elem,UINT8* value,UINT32* valuelen)
 	{
@@ -543,6 +566,7 @@ SINT32 getDOMElementValue(const DOM_Node& elem,UINT8* value,UINT32* valuelen)
 		return E_SUCCESS;
 	}
 
+
 SINT32 getDOMElementValue(const DOM_Element& elem,UINT32* value)
 	{
 		ASSERT(value!=NULL,"Value is null");
@@ -555,6 +579,17 @@ SINT32 getDOMElementValue(const DOM_Element& elem,UINT32* value)
 		
 		return E_SUCCESS;
 	}
+SINT32 getDOMElementValue(const DOM_Element& elem,UINT32& value, UINT32 defaultValue)
+{
+	UINT32 v;
+	if(getDOMElementValue(elem,&v)!=E_SUCCESS)
+		{
+		value=defaultValue;
+		}
+	else
+	value=v;	
+	return E_SUCCESS;
+}
 
 SINT32 getDOMElementValue(const DOM_Element& elem, UINT64 &value)
 {
@@ -667,6 +702,7 @@ SINT32 decodeXMLEncryptedKey(UINT8* key,UINT32* keylen, DOM_Node & root,CAASymCi
 	}
 
 /** The resulting encrypted xml struct is as follows:
+@verbatim
 	* <EncryptedData Type='http://www.w3.org/2001/04/xmlenc#Element'>
 	*		<EncryptionMethod Algorithm="http://www.w3.org/2001/04/xmlenc#aes128-cbc"/>
 	*   <ds:KeyInfo xmlns:ds='http://www.w3.org/2000/09/xmldsig#'>
@@ -681,6 +717,7 @@ SINT32 decodeXMLEncryptedKey(UINT8* key,UINT32* keylen, DOM_Node & root,CAASymCi
 	*			<CipherValue>...</CipherValue>
 	*		</CipherData>
 	*	</EncryptedData>
+	*@endverbatim
 	*/
 SINT32 encryptXMLElement(DOM_Node node, CAASymCipher* pRSA)
 	{
@@ -742,7 +779,56 @@ SINT32 encryptXMLElement(DOM_Node node, CAASymCipher* pRSA)
 			}
 		return E_SUCCESS;
 	}
+#endif //ONLY_LOCAL_PROXY
 
+/** Encrypts an XML-Element by wrapping it with:
+*@verbatim
+	* \<EncryptedData Type='http://www.w3.org/2001/04/xmlenc#Element'>
+	*		\<EncryptionMethod Algorithm="http://www.w3.org/2001/04/xmlenc#aes128-cbc"/>
+	*   \<ds:KeyInfo xmlns:ds='http://www.w3.org/2000/09/xmldsig#'>
+	*			\<EncryptedKey>
+	*				\<EncryptionMethod Algorithm="http://www.w3.org/2001/04/xmlenc#rsa-oaep-mgf1p"/>
+	*				\<CipherData\>
+	*					\<CipherValue>...\</CipherValue>					
+	*				\</CipherData>
+	*			\</EncryptedKey>
+	*		\</ds:KeyInfo>
+	*		\<CipherData>
+	*			\<CipherValue>...\</CipherValue>
+	*		\</CipherData>
+	*	\</EncryptedData>
+	*@endverbatim
+	@return a buffer with the encyrpted element (the caller is responsible for freeing it), or NULL
+*/
+UINT8* encryptXMLElement(UINT8* inbuff,UINT32 inlen,UINT32& outlen,CAASymCipher* pRSA)
+	{
+		const char* XML_ENC_TEMPLATE="<EncryptedData><ds:KeyInfo><EncryptedKey><CipherData><CipherValue>%s</CipherValue></CipherData></EncryptedKey></ds:KeyInfo><CipherData><CipherValue>%s</CipherValue></CipherData></EncryptedData>";
+		UINT8 key[32];
+		getRandom(key,32);
+		UINT8 buff[1000];
+		UINT32 bufflen=255;
+		pRSA->encryptOAEP(key,32,buff,&bufflen);
+		UINT8 keyoutbuff[1000];
+		UINT32 keyoutbufflen=255;
+		CABase64::encode(buff,bufflen,keyoutbuff,&keyoutbufflen);
+		keyoutbuff[keyoutbufflen]=0;
+		CASymCipher *pSymCipher=new CASymCipher();
+		pSymCipher->setKey(key,true);
+		pSymCipher->setIVs(key+16);
+		UINT8* msgoutbuff=new UINT8[10000];
+		UINT32 msgoutbufflen=10000;
+		pSymCipher->encrypt1CBCwithPKCS7(inbuff,inlen,msgoutbuff,&msgoutbufflen);
+		UINT8* encmsgoutbuff=new UINT8[10000];
+		UINT32 encmsgoutbufflen=10000;
+		CABase64::encode(msgoutbuff,msgoutbufflen,encmsgoutbuff,&encmsgoutbufflen);
+		encmsgoutbuff[encmsgoutbufflen]=0;
+		sprintf((char*)msgoutbuff,XML_ENC_TEMPLATE,keyoutbuff,encmsgoutbuff);
+		outlen=strlen((char*)msgoutbuff);
+		delete[] encmsgoutbuff;
+		return msgoutbuff;
+	}
+
+#ifndef ONLY_LOCAL_PROXY
 SINT32 decryptXMLElement(DOM_Node node, CAASymCipher* pRSA)
 	{
 		DOM_Document doc=node.getOwnerDocument();
@@ -810,6 +896,7 @@ SINT32 decryptXMLElement(DOM_Node node, CAASymCipher* pRSA)
 			parent.replaceChild(elemPlainRoot,node);	
 		return E_SUCCESS;
 	}
+#endif //ONLY_LOCAL_PROXY
 
 UINT8* readFile(UINT8* name,UINT32* size)
 	{
