@@ -26,6 +26,7 @@ IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISI
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 */
 #include "../StdAfx.h"
+#ifndef ONLY_LOCAL_PROXY
 #include "CAFirstMixA.hpp"
 #include "../CAThread.hpp"
 #include "../CASingleSocketGroup.hpp"
@@ -180,7 +181,7 @@ SINT32 CAFirstMixA::loop()
 												if(ret == 3) 
 													{
 														// this jap is evil! terminate connection and add IP to blacklist
-														CAMsg::printMsg(LOG_DEBUG, "Detected evil Jap.. closing connection! Removing IP..\n\n", ret);
+														CAMsg::printMsg(LOG_DEBUG, "CAFirstMixA: Detected evil Jap.. closing connection! Removing IP..\n\n", ret);
 														fmChannelListEntry* pEntry;
 														pEntry=m_pChannelList->getFirstChannelForSocket(pMuxSocket);
 														while(pEntry!=NULL)
@@ -614,3 +615,4 @@ NEXT_USER:
 #endif //!MIX_NEW_TYP
 		return E_UNKNOWN;
 	}
+#endif //ONLY_LOCAL_PROXY
