@@ -32,32 +32,34 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMA
 
 #include "CACmdLnOptions.hpp"
 #include "CAMsg.hpp"
-#include "CAMix.hpp"
-#include "CAMiddleMix.hpp"
-#include "TypeA/CAFirstMixA.hpp"
-#include "TypeA/CALastMixA.hpp"
 #include "CALocalProxy.hpp"
-#include "xml/DOM_Output.hpp"
-#ifdef LOG_CRIME
-#include "tre/regex.h"
+#include "CAQueue.hpp"
+#ifndef ONLY_LOCAL_PROXY
+	#include "xml/DOM_Output.hpp"
+	#include "CAMix.hpp"
+	#ifdef LOG_CRIME
+		#include "tre/regex.h"
+	#endif
+	#ifdef NEW_MIX_TYPE
+		/* use TypeB mixes */
+		#include "TypeB/CAFirstMixB.hpp"
+		#include "TypeB/CALastMixB.hpp"
+	#else
+		#include "TypeA/CAFirstMixA.hpp"
+		#include "TypeA/CALastMixA.hpp"
+	#endif
+	#include "CAMiddleMix.hpp"
+	#include "CALogPacketStats.hpp"
+	#include "CATLSClientSocket.hpp"
+
+// The Mix....
+CAMix* pMix=NULL;
 #endif
-#ifdef NEW_MIX_TYPE
-	#include "TypeB/CAFirstMixB.hpp"
-	#include "TypeB/CALastMixB.hpp"
-#endif
-#include "CALogPacketStats.hpp"
-#include "CATLSClientSocket.hpp"
-//#include "CAPayment.hpp"
-//#ifdef _WIN32
-//HANDLE hEventThreadEnde;
-//#endif
 CACmdLnOptions options;
 
 //Global Locks required by OpenSSL-Library
 CAMutex* pOpenSSLMutexes;
 
-// The Mix....
-CAMix* pMix=NULL;
 
 
 typedef struct
@@ -107,11 +109,12 @@ void signal_interrupt( int)
 		exit(0);
 	}
 
+#ifndef ONLY_LOCAL_PROXY
 void signal_hup(int)
 	{
 		options.reread(pMix);
 	}
-
+#endif
 
 ///Callbackfunction for looking required by OpenSSL
 void openssl_locking_callback(int mode, int type, char * /*file*/, int /*line*/)
@@ -174,11 +177,15 @@ void checkSizesOfBaseTypes()
 /**do necessary initialisations of libraries etc.*/
 void init()
 	{
+#ifndef ONLY_LOCAL_PROXY
 		XMLPlatformUtils::Initialize();
+#endif
 		OpenSSL_add_all_algorithms();
 		pOpenSSLMutexes=new CAMutex[CRYPTO_num_locks()];
 		CRYPTO_set_locking_callback((void (*)(int,int,const char *,int))openssl_locking_callback);
+#ifndef ONLY_LOCAL_PROXY
 		SSL_library_init();
+#endif
 		CAMsg::init();
 		CASocketAddrINet::init();
 		//startup
@@ -296,7 +303,7 @@ No data is transmitted during this step.
 
 \li Step 3: Mix 3 sends its public key  to Mix 2 (signed with its signature key).
 See \ref XMLInterMixInitSendFromLast "[XML]" for a description of the XML struct send in this step.
-The <Nonce/> is a random byte string of length=16 (chosen by Mix 3) used for replay detection.
+The \<Nonce/\> is a random byte string of length=16 (chosen by Mix 3) used for replay detection.
 
 \li Step 4: Mix 2 generates and sends a symmetric key to Mix 3, encrypted with the public key
 of Mix 3, signed by Mix 2.
@@ -306,7 +313,7 @@ The first 16 bytes a used as a key for
 inter-mix encryption of packets send from Mix \e n-1 to Mix \e n (e.g. upstream direction).
 The next 16 byte are used as IV for this cipher. The next 16 bytes are used as
 key for downstream direction and the last 16 bytes are the IV for this cipher.
-To detect replay attacks Mix 3 checks if the <Nonce/> element sent from Mix 2 is equal to SHA1(<Nonce/> chosen by Mix 3 in Step 3).
+To detect replay attacks Mix 3 checks if the \<Nonce/\> element sent from Mix 2 is equal to SHA1(\<Nonce\/> chosen by Mix 3 in Step 3).
 
 \li Step 5,6,7: This steps are equal to step 2,3,4. Mix 1 establishes a TCP/IP-connection
 with Mix 2. Mix 2 send it publick key to Mix 1 and Mix 1 generates and sends
@@ -367,7 +374,9 @@ See \ref XMLMixCascadeStatus "[XML]" for a description of the XML struct send.
 
 int main(int argc, const char* argv[])
 	{
+#ifndef ONLY_LOCAL_PROXY
 		pMix=NULL;
+#endif
 		int i;
 		SINT32 maxFiles,ret;
 #if defined(HAVE_CRTDBG)
@@ -416,7 +425,7 @@ int main(int argc, const char* argv[])
 			}
 #endif
 #ifdef LOG_CRIME
-			testTre();
+//			testTre();
 #endif
 
 #ifdef _DEBUG
@@ -440,7 +449,6 @@ int main(int argc, const char* argv[])
 			CAMsg::printMsg(LOG_CRIT,"Error: Cannot parse configuration file!\n");
 			goto EXIT;
 		}
-
 		if(!(options.isFirstMix()||options.isMiddleMix()||options.isLastMix()||options.isLocalProxy()))
 			{
 				CAMsg::printMsg(LOG_CRIT,"You must specifiy, which kind of Mix you want to run!\n");
@@ -473,6 +481,10 @@ int main(int argc, const char* argv[])
 					chdir("/");
 					umask(0);
 				#endif
+			 // Close out the standard file descriptors 
+        close(STDIN_FILENO);
+        close(STDOUT_FILENO);
+        close(STDERR_FILENO);			
 			}
 #endif
 		if(options.getLogDir((UINT8*)buff,255)==E_SUCCESS)
@@ -482,7 +494,9 @@ int main(int argc, const char* argv[])
 				else
 					CAMsg::setLogOptions(MSG_FILE);
 			}
+#ifndef ONLY_LOCAL_PROXY
 		ret=CAMsg::openEncryptedLog();
+#endif
 #ifdef LOG_CRIME
 		if(ret!=E_SUCCESS)
 			{
@@ -497,7 +511,7 @@ int main(int argc, const char* argv[])
 #endif
 
 
-#ifdef _DEBUG
+#if defined (_DEBUG) &&!defined(ONLY_LOCAL_PROXY)
 		//		CADatabase::test();
 		if(CAQueue::test()!=E_SUCCESS)
 			CAMsg::printMsg(LOG_CRIT,"CAQueue::test() NOT passed! Exiting\n");
@@ -547,9 +561,6 @@ int main(int argc, const char* argv[])
 
 		CAMsg::printMsg(LOG_INFO,"Anon proxy started!\n");
 		CAMsg::printMsg(LOG_INFO,MIX_VERSION_INFO);
-#ifdef _DEBUG
-		sockets=0;
-#endif
 
 
 #ifndef _WIN32
@@ -558,11 +569,12 @@ int main(int argc, const char* argv[])
 	#else
 			signal(SIGPIPE,SIG_IGN);
 	#endif
+	#ifndef ONLY_LOCAL_PROXY
 		struct sigaction newAction;
 		newAction.sa_handler=signal_hup;
 		newAction.sa_flags=0;
 		sigaction(SIGHUP,&newAction,NULL);
-		//signal(SIGHUP,signal_hup);
+	#endif
 #endif
 		signal(SIGINT,signal_interrupt);
 		signal(SIGTERM,signal_term);
@@ -603,7 +615,11 @@ int main(int argc, const char* argv[])
 		if(options.isLocalProxy())
 			{
 				#ifndef NEW_MIX_TYPE
-					pMix=new CALocalProxy();
+					CALocalProxy* pProxy=new CALocalProxy();
+					CAMsg::printMsg(LOG_INFO,"Starting LocalProxy...\n");
+					if(pProxy->start()!=E_SUCCESS)
+						CAMsg::printMsg(LOG_CRIT,"Error during MIX-Startup!\n");
+					delete pProxy;
 				#else
 					CAMsg::printMsg(LOG_CRIT,"Compiled without LocalProxy support!\n");
 					goto EXIT;
@@ -619,6 +635,13 @@ int main(int argc, const char* argv[])
 				//		goto EXIT;
 				//	}
 				//else
+#ifndef ONLY_LOCAL_PROXY
+			SINT32 s32MaxSockets=CASocket::getMaxOpenSockets();
+			CAMsg::printMsg(LOG_CRIT,"Max Number of sockets we can open: %i\n",s32MaxSockets);
+			if(s32MaxSockets>100&&s32MaxSockets<10000)
+				{
+				CASocket::setMaxNormalSockets(s32MaxSockets-10);
+				}
 				if(options.isFirstMix())
 					{
 						CAMsg::printMsg(LOG_INFO,"I am the First MIX..\n");
@@ -639,15 +662,69 @@ int main(int argc, const char* argv[])
 						#else
 							pMix=new CALastMixB();
 						#endif
+#else
+				CAMsg::printMsg(LOG_ERR,"this Mix is compile to work only as local proxy!\n");
+				goto EXIT;
+#endif
 			}
+#ifndef ONLY_LOCAL_PROXY
+#ifndef DYNAMIC_MIX
 	  CAMsg::printMsg(LOG_INFO,"Starting MIX...\n");
 		if(pMix->start()!=E_SUCCESS)
 			CAMsg::printMsg(LOG_CRIT,"Error during MIX-Startup!\n");
+#else
+	//*sk13* Nasty think top be removed after test...
+	if(!options.isLastMix())
+		{
+		  #define LOG_OUT "/home/tudresden_dynamicmix/DynMix/"
+			options.setLogDir((UINT8*)LOG_OUT,strlen(LOG_OUT));
+			CAMsg::setLogOptions(MSG_FILE);
+		}
+    /* LERNGRUPPE */
+while(true) 
+{
+	CAMsg::printMsg(LOG_INFO,"Starting MIX...\n");
+	if(pMix->start()!=E_SUCCESS)
+	{
+		/** @todo Hmm, maybe we could remain running, but that may well result in an endless running loop eating the cpu */
+		CAMsg::printMsg(LOG_CRIT,"Error during MIX-Startup!\n");
+		goto EXIT;
+	}
+
+	/* If we got here, the mix should already be reconfigured, so we only need a new instance */
+	if(pMix!=NULL)
+		delete pMix;
+
+	if(options.isFirstMix())
+	{
+		CAMsg::printMsg(LOG_INFO,"I am now the First MIX..\n");
+#if !defined(NEW_MIX_TYPE)
+            pMix=new CAFirstMixA();
+#else
+            pMix=new CAFirstMixB();
+#endif
+	}
+	else if(options.isMiddleMix())
+	{
+		CAMsg::printMsg(LOG_INFO,"I am now a Middle MIX..\n");
+		pMix=new CAMiddleMix();
+	}
+	else
+	{
+		/* Reconfiguration of a last mix?! Not really...*/
+		CAMsg::printMsg( LOG_ERR, "Tried to reconfigure a former first/middle-Mix to a LastMix -> impossible!\n");
+		goto EXIT;
+	}
+}
+#endif //DYNAMIC_MIX
+#endif //ONLY_LOCAL_PROXY
 EXIT:
 //		delete pRTT;
+#ifndef ONLY_LOCAL_PROXY
 		if(pMix!=NULL)
 			delete pMix;
-//		CASocketAddrINet::destroy();
+#endif
+		//		CASocketAddrINet::destroy();
 		#ifdef _WIN32
 			WSACleanup();
 		#endif
@@ -660,7 +737,9 @@ EXIT:
 //XML Cleanup
 		//Note: We have to destroy all XML Objects and all objects that uses XML Objects BEFORE
 		//we terminate the XML lib!
+#ifndef ONLY_LOCAL_PROXY
 		XMLPlatformUtils::Terminate();
+#endif //ONLY_LOCAL_PROXY
 		CAMsg::printMsg(LOG_CRIT,"Terminating Programm!\n");
 		CAMsg::cleanup();
 #if defined(HAVE_CRTDBG)
