@@ -27,7 +27,7 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMA
 */
 #ifndef __TYEDEFS__
 #define __TYEDEFS__
-enum NetworkType {UNKNOWN_NETWORKTYPE,RAW_TCP,RAW_UNIX,SSL_TCP,SSL_UNIX};
+enum NetworkType {UNKNOWN_NETWORKTYPE,RAW_TCP,RAW_UNIX,SSL_TCP,SSL_UNIX, HTTP_TCP};
 
 typedef UINT32 HCHANNEL;
 #define MIX_PAYLOAD_HTTP  0 
@@ -56,12 +56,12 @@ typedef UINT32 HCHANNEL;
 	#define NEW_FLOW_CONTROL_FLAG 0x8000
 #endif
 	
-#ifndef NEW_MIX_TYPE
+#define CONNECTION_ERROR_FLAG 0x4000
 
-	#define DATA_SIZE 			992
-	#define PAYLOAD_SIZE 		989
+#define DATA_SIZE 			992
+#define PAYLOAD_SIZE 		989
 
-	#if (defined(WIN32) ||defined(__sgi))&&!defined(__GNUC__)
+#if (defined(WIN32) ||defined(__sgi))&&!defined(__GNUC__)
 		#pragma pack( push, t_MixPacket )
 		#pragma pack(1)
 		struct t_MixPacketPayload
@@ -81,14 +81,13 @@ typedef UINT32 HCHANNEL;
 					};
 			};
 		#pragma pack( pop, t_MixPacket )
-	#else
+#else
 		struct t_MixPacketPayload
 			{
 				UINT16 len;
 				UINT8 type;
 				UINT8 data[PAYLOAD_SIZE];
 			} __attribute__ ((__packed__));
-
 		struct t_MixPacket
 			{
 				HCHANNEL channel;
@@ -99,53 +98,7 @@ typedef UINT32 HCHANNEL;
 						struct t_MixPacketPayload payload;
 					};
 			} __attribute__ ((__packed__)); // MUXPACKET __attribute__ ((__packed__));
-	#endif //WIN32 
-#else //NEW_MIX_TYPE
-
-	#define DATA_SIZE 			994
-	#define PAYLOAD_SIZE 		979
-
-	#if defined(WIN32) ||defined(__sgi)
-		#pragma pack( push, t_MixPacket )
-		#pragma pack(1)
-		struct t_MixPacket
-			{
-				HCHANNEL channel;
-				union
-					{
-						UINT8		data[DATA_SIZE];
-						struct t_MixPacketPayload
-							{
-								UINT8 linkid[8];
-								UINT8 seq[4];
-								UINT8 type;
-								UINT16 len;
-								UINT8 data[PAYLOAD_SIZE];
-						} payload;
-					};
-			};
-		#pragma pack( pop, t_MixPacket )
-	#else
-		struct t_MixPacketPayload
-			{
-				UINT8 linkid[8];
-				UINT8 seq[4];
-				UINT8 type;
-				UINT16 len;
-				UINT8 data[PAYLOAD_SIZE];
-			} __attribute__ ((__packed__));
-
-		struct t_MixPacket
-			{
-				HCHANNEL channel;
-				union
-					{
-						UINT8		data[DATA_SIZE];
-						struct t_MixPacketPayload payload;
-					};
-			} __attribute__ ((__packed__)); // MUXPACKET __attribute__ ((__packed__));
-	#endif //WIN32 
-#endif //NEW_MIX_TYPE
+#endif //WIN32 
 
 typedef t_MixPacket MIXPACKET;
 
@@ -221,6 +174,8 @@ typedef struct t_mix_parameters tMixParameters;
 /** we have sent one request for an accountcertificate */
 #define AUTH_SENT_ACCOUNT_REQUEST 0x100
 
+#define AUTH_HARD_LIMIT_REACHED 0x200
+
 /** the user tried to fake something */
 #define AUTH_FAKE 0x400
 
@@ -233,6 +188,8 @@ typedef struct t_mix_parameters tMixParameters;
 /** a fatal error occured earlier */
 #define AUTH_FATAL_ERROR 0x2000
 
+
+
 class CASignature;
 class CAAccountingControlChannel;
 /**
@@ -277,6 +234,9 @@ struct t_accountinginfo
 	/** Flags, see above AUTH_* */
 	UINT32 authFlags;
 
+	/** timestamp when last HardLimit was reached */
+	SINT32 lastHardLimitSeconds;
+
 	/** timestamp when last PayRequest was sent */
 	SINT32 lastRequestSeconds;
 	
